﻿namespace BNFUP.Interfaces
{
    public delegate void ObjectCreatedHandler(ICompilableObjectGenerator item);
    /// <summary>
    /// Interface implemented by the rule items which generate objects in the compiler side
    /// </summary>
    public interface ICompilableObjectGenerator
    {
        /// <summary>
        /// Raised when the object is created
        /// </summary>
        event ObjectCreatedHandler OnCreateObject;
        /// <summary>
        /// Raised when the object is successful built
        /// </summary>
        event ObjectCreatedHandler OnCompleteObject;
        /// <summary>
        /// Raised to discard the created object
        /// </summary>
        event ObjectCreatedHandler OnDiscardObject;
        /// <summary>
        /// This is the key passed to the object factory to create the object
        /// </summary>
        string CompilableType { get; set; }
        /// <summary>
        /// Object generator
        /// </summary>
        ICompilableObjectFactory Factory { get; set; }
        /// <summary>
        /// Current object managed by the rule item
        /// </summary>
        ICompilableObject Object { get; }
        /// <summary>
        /// The compiler calls this method to set up the object in the rule item
        /// </summary>
        /// <param name="obj">The most recent version of the current compiler object</param>
        void UpdateObject(ICompilableObject obj);  
    }
}
