﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using BNFUP.Rules;

namespace BNFUP.Interfaces
{
    public enum EditActions
    {
        Delete, Cut, Copy, Paste, Enlist, Extract, Free, Up, Down, New, Simplify, Alternatives
    }
    /// <summary>
    /// This interface is used to edit the rule item
    /// </summary>
    public interface IEditableItem
    {
        /// <summary>
        /// Indicates that the item can be copied to the clipboard
        /// </summary>
        bool CanCopy { get; }
        /// <summary>
        /// Index of the item in parent list when edited
        /// </summary>
        int ItemIndex { get; set; }
        /// <summary>
        /// Return all the available edit options to set up a context menu
        /// </summary>
        /// <param name="parent">Parent item</param>
        /// <param name="gparent">Grand parent item</param>
        /// <param name="clipboard">Clipboard implementation</param>
        /// <returns>Menu options enumeration</returns>
        IEnumerable<ToolStripItem> EditOptions(IEditableItem parent, IEditableItem gparent, IRuleItemClipboard clipboard);
        /// <summary>
        /// Execute the selected option
        /// </summary>
        /// <param name="action">Menu option selected</param>
        /// <param name="parent">Parent item</param>
        /// <param name="gparent">Grand parent item</param>
        /// <param name="rules">Rule table</param>
        /// <param name="clipboard">Clipboard implementation</param>
        /// <returns>True if rule refresh needed</returns>
        bool PerformAction(ToolStripItem action, IEditableItem parent, IEditableItem gparent, RuleTable rules, IRuleItemClipboard clipboard);
        /// <summary>
        /// Add a new child to the object
        /// </summary>
        /// <param name="item">New item to add</param>
        void AddChild(IRuleItem item);
        /// <summary>
        /// Remove a child from the object
        /// </summary>
        /// <param name="item">Index of the item to remove</param>
        void RemoveChild(int item);
        /// <summary>
        /// Replace one child item with another
        /// </summary>
        /// <param name="item1">Index of the old item to replace</param>
        /// <param name="item2">New item for replace</param>
        void ReplaceChild(int item1, IRuleItem item2);
        /// <summary>
        /// Insert child item after another
        /// </summary>
        /// <param name="item1">Index of the existent child</param>
        /// <param name="item2">New item to inert after item1</param>
        void InsertAfter(int item1, IRuleItem item2);
        /// <summary>
        /// Move a child backward in the child list
        /// </summary>
        /// <param name="item">Index of child item to move up</param>
        void MoveUp(int item);
        /// <summary>
        /// Move a child forward in the child list
        /// </summary>
        /// <param name="item">Index of child item to move down</param>
        void MoveDown(int item);
        /// <summary>
        /// Validate the type of a possible child.
        /// </summary>
        /// <param name="t">Child type</param>
        /// <param name="generic">Indicates if the child is new or an existing one</param>
        /// <returns>True if the child type is accepted</returns>
        bool ValidateType(Type t, bool generic);
    }
}
