﻿using System;
using System.Drawing;
using System.Text.RegularExpressions;
using BNFUP.Properties;
using BNFUP.Attributes;

namespace BNFUP.Rules
{
    [Serializable]
    public class CharacterSet : ItemBase
    {
        private string _chSet = "";
        private bool _scape = true;

        public CharacterSet()
        {
        }
        public CharacterSet(string chset)
        {
            ChSet = chset;
        }
        public override string ID
        {
            get
            {
                return "chset:" + _chSet;
            }
        }
        [CategoryGlobal("CAT_SYNTAX")]
        [DisplayNameGlobal("DIS_SCAPE")]
        [DescriptionGlobal("DES_SCAPE")]
        public bool Scape
        {
            get
            {
                return _scape;
            }
            set
            {
                _scape = value;
            }
        }
        [CategoryGlobal("CAT_SYNTAX")]
        [DisplayNameGlobal("DIS_CHSET")]
        [DescriptionGlobal("DES_CHSET")]
        public string ChSet
        {
            get
            {
                return _chSet;
            }
            set
            {
                if (_chSet != value)
                {
                    ValidateID("chset:" + value);
                }
                Regex regv = new Regex("[" + (_scape ? Regex.Escape(value) : value) + "]");
                _chSet = value;
            }
        }
        #region IRuleItem
        public override string Description
        {
            get
            {
                return Resources.DES_CHARSET + ": " + _chSet;
            }
        }
        public override bool Matches(Parser parser)
        {
            return parser.TryRegex("^[" + (_scape ? Regex.Escape(_chSet) : ChSet) + "]");
        }
        public override void Build(Parser parser)
        {
            _text = "";
            do
            {
                _text += parser.GetNext("^[" + (_scape ? Regex.Escape(_chSet) : ChSet) + "]");
            } while (!parser.EOF && Matches(parser));
            base.Build(parser);
        }
        #endregion
        #region IItemDrawing
        public override SizeF Measure(Graphics gr, Font fextra, Font fmain)
        {
            SizeF szi = gr.MeasureString("{ ", fextra);
            SizeF sz = gr.MeasureString(_chSet, fmain);
            SizeF szf = gr.MeasureString(" }", fextra);
            float wo = 0f;
            if (Optional)
            {
                wo = gr.MeasureString("[ ", fextra).Width + gr.MeasureString(" ]", fextra).Width;
            }
            return new SizeF(szi.Width + sz.Width + szf.Width + wo, szi.Height);
        }
        public override void Draw(Graphics gr, float x, float y, Font fextra, Font fmain)
        {
            SizeF szi = gr.MeasureString("{ ", fextra);
            SizeF sz = gr.MeasureString(_chSet, fmain);
            Brush bc = new SolidBrush(ForeColor);
            if (Optional)
            {
                gr.DrawString("[ ", fextra, Brushes.Black, x, y);
                x += gr.MeasureString("[ ", fextra).Width;
            }
            gr.DrawString("{ ", fextra, Brushes.Black, x, y);
            x += szi.Width;
            gr.DrawString(_chSet, fmain, bc, x, y);
            x += sz.Width;
            gr.DrawString(" }", fextra, Brushes.Black, x, y);
            if (Optional)
            {
                gr.DrawString(" ]", fextra, Brushes.Black, x, y);
            }
            bc.Dispose();
        }
        #endregion
        #region IEditableItem
        public override bool CanCopy
        {
            get
            {
                return !string.IsNullOrEmpty(_chSet);
            }
        }
        #endregion
    }
}
