﻿using System;
using System.Drawing;
using BNFUP.Properties;
using BNFUP.Attributes;

namespace BNFUP.Rules
{
    [Serializable]
    public class Token : ItemBase
    {
        private string _name = "";
        public Token()
        {
        }
        public Token(string name)
        {
            _name = name;
        }
        public override string ToString()
        {
            if (_name == "'")
            {
                return "''''";
            }
            return "'" + _name + "'";
        }
        public override string ID
        {
            get
            {
                return "token:" + _name;
            }
        }
        [CategoryGlobal("CAT_SYNTAX")]
        [DisplayNameGlobal("DIS_TOKENNAME")]
        [DescriptionGlobal("DES_TOKENNAME")]
        public string Name
        {
            get
            {
                return _name;
            }
            set
            {
                if (_name != value)
                {
                    ValidateID("token:" + value);
                }
                _name = value;
            }
        }
        #region IRuleItem
        public override string Description
        {
            get
            {
                return string.Join(Resources.SEP_DESCRIPTOR, new string[] { Resources.DES_TOKEN, Name });
            }
        }
        public override bool Matches(Parser parser)
        {
            return parser.TryParse(_name);
        }
        public override void Build(Parser parser)
        {
            _text = "";
            parser.Parse(_name);
            _text = _name;
            base.Build(parser);
        }
        #endregion
        #region IItemDrawing
        public override SizeF Measure(Graphics gr, Font fextra, Font fmain)
        {
            SizeF szn = gr.MeasureString(_name, fmain);
            float wo = 0f;
            if (Optional)
            {
                wo = gr.MeasureString("[ ", fextra).Width + gr.MeasureString(" ]", fextra).Width;
            }
            return new SizeF(szn.Width + wo, szn.Height);
        }
        public override void Draw(Graphics gr, float x, float y, Font fextra, Font fmain)
        {
            Brush bc = new SolidBrush(ForeColor);
            if (Optional)
            {
                gr.DrawString("[ ", fextra, Brushes.Black, x, y);
                x += gr.MeasureString("[ ", fextra).Width;
            }
            gr.DrawString(_name, fmain, bc, x, y);
            x+= gr.MeasureString(_name, fmain).Width;
            if (Optional)
            {
                gr.DrawString(" ]", fextra, Brushes.Black, x, y);
            }
            bc.Dispose();
        }
        #endregion
        #region IEditableItem
        public override bool CanCopy
        {
            get
            {
                return !string.IsNullOrEmpty(_name);
            }
        }
        #endregion
    }
}
