﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CellularAutomata.Interfaces;
using CellularAutomata.Properties;

namespace CellularAutomata.Automata
{
    public class BasicCAOptimizer : ICAOptimizer
    {
        private Dictionary<int, TransitionTable> _states = new Dictionary<int, TransitionTable>();
        
        public BasicCAOptimizer()
        {
            MaxCapacity = 1000;
        }
        public override string ToString()
        {
            return UIResources.OPN_BASIC;
        }
        #region ICAOptimizer
        public int MaxCapacity { get; set; }
        public void AddKnownTransition(int from, int[] neighborhood, int to)
        {
            if (_states.ContainsKey(from))
            {
                _states[from].AddTransition(neighborhood, to);
            }
            else
            {
                _states[from] = new TransitionTable(neighborhood, to, MaxCapacity);
            }
        }
        public int GetTransition(int from, int[] neighborhood)
        {
            if (_states.ContainsKey(from))
            {
                return _states[from].GetTransition(neighborhood);
            }
            return -1;
        }
        #endregion
    }
    internal class TransitionTable
    {
        private Dictionary<string, int> _table = new Dictionary<string, int>();
        private List<string> _history = new List<string>();
        private int _maxEntries = 1000;

        public TransitionTable()
        {
        }
        public TransitionTable(int[] neighborhood, int to)
        {
            AddTransition(neighborhood, to);
        }
        public TransitionTable(int[] neighborhood, int to, int maxentries)
        {
            AddTransition(neighborhood, to);
            _maxEntries = maxentries;
        }
        public void AddTransition(int[] neighborhood, int to)
        {
            string k = Transition(neighborhood);
            _table[k] = to;
            PingTransition(k);
        }
        public int GetTransition(int[] neighborhood)
        {
            string k = Transition(neighborhood);
            if (_table.ContainsKey(k))
            {
                PingTransition(k);
                return _table[k];
            }
            return -1;
        }
        private string Transition(int[] neighborhood)
        {
            List<int> ln = new List<int>(neighborhood);
            string s = "";
            string pc = "";
            foreach (int n in ln)
            {
                s += pc + n.ToString();
                pc = ";";
            }
            return s;
        }
        private void PingTransition(string k)
        {
            _history.Remove(k);
            _history.Insert(0, k);
            if (_history.Count > _maxEntries)
            {
                _table.Remove(_history[_history.Count - 1]);
                _history.RemoveAt(_history.Count - 1);
            }
        }
    }
}
