﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CellularAutomata.Interfaces;
using CellularAutomata.Properties;

namespace CellularAutomata.Automata
{
    [Serializable]
    public class BasicCell : ICACell
    {
        private int[] _neighborhood = null;
        private int[] _stateBuffer = new int[2];
        private int _ixBuffer = 0;

        public BasicCell()
        {
        }
        public override string ToString()
        {
            return UIResources.CEN_BASIC;
        }
        #region ICACell
        public int ActualState
        {
            get
            {
                return _stateBuffer[_ixBuffer];
            }
            set
            {
                _ixBuffer = 0;
                _stateBuffer[_ixBuffer] = value;
            }
        }
        public int NextState
        {
            get
            {
                return _stateBuffer[_ixBuffer ^ 1];
            }
            set
            {
                _stateBuffer[_ixBuffer ^ 1] = value;
            }
        }
        public IEnumerable<int> Neighborhood
        {
            get
            {
                foreach (int n in _neighborhood)
                {
                    yield return n;
                }
            }
        }
        public int NeighborCount
        {
            get
            {
                if (_neighborhood != null)
                {
                    return _neighborhood.Length;
                }
                return -1;
            }
        }
        public void AddNeighbours(int[] ixn)
        {
            _neighborhood = ixn;
        }
        public int[] GetNeighborhood()
        {
            if (_neighborhood != null)
            {
                int[] nc = new int[_neighborhood.Length];
                _neighborhood.CopyTo(nc, 0);
                return nc;
            }
            return null;
        }
        public int GetNeighbour(int ix)
        {
            return _neighborhood[ix];
        }
        public void ClearNeighborhood()
        {
            _neighborhood = null;
        }
        public int GetState(int step)
        {
            _ixBuffer = step & 1;
            return _stateBuffer[_ixBuffer];
        }
        #endregion
    }
}
