﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CellularAutomata.Controls
{
    public partial class Grid2DCAEditor : UserControl, IComparer<int>
    {
        private const int sqs = 20;

        private List<int> _left = new List<int>();
        private List<int> _right = new List<int>();
        private Size _length = new Size(0, 0);

        public Grid2DCAEditor()
        {
            InitializeComponent();
        }
        #region IComparer<int>
        public int Compare(int v1, int v2)
        {
            if (v1 == v2)
            {
                return 0;
            }
            if ((v1 == 0) || (v2 == 0) || ((v1 > 0) && (v2 > 0)))
            {
                return v1 - v2;
            }
            if ((v1 < 0) && (v2 < 0))
            {
                int cntr = (_length.Width - 1) / 2;
                int cntl = (_length.Width - 1) - cntr;
                int y1 = (v1 - cntl) / _length.Width;
                int y2 = (v2 - cntl) / _length.Width;
                int x1 = v1 - (y1 * _length.Width);
                int x2 = v2 - (y2 * _length.Width);
                if ((y1 == 0) && (y2 == 0))
                {
                    return x2 - x1;
                }
                if (y1 == y2)
                {
                    return x1 - x2;
                }
                if (y1 == 0)
                {
                    return 1;
                }
                if (y2 == 0)
                {
                    return -1;
                }
                return y2 - y1;
            }
            if (v1 < 0)
            {
                return -1;
            }
            return 1;
        }
        #endregion
        public event EventHandler OnChangeNeighborhood = null;
        public Size Length
        {
            get
            {
                return _length;
            }
            set
            {
                _length = value;
                if ((_length.Width > 0) && (_length.Height > 0))
                {
                    int cntr = (_length.Width - 1) / 2;
                    int cntl = (_length.Width - 1) - cntr;
                    int cntd = (_length.Height - 1) / 2;
                    int cntu = (_length.Height - 1) - cntd;
                    for (int ix = _left.Count - 1; ix >= 0; ix--)
                    {
                        if (((_left[ix] / _length.Width) < -cntu) || ((_left[ix] % _length.Width) < -cntl))
                        {
                            _left.RemoveAt(ix);
                        }
                    }
                    for (int ix = _right.Count - 1; ix >= 0; ix--)
                    {
                        if (((_right[ix] / _length.Width) > cntd) || ((_right[ix] % _length.Width) > cntr))
                        {
                            _right.RemoveAt(ix);
                        }
                    }
                    SetHScroll();
                    SetVScroll();
                    Refresh();
                }
            }
        }
        public int[] Neighborhood
        {
            get
            {
                List<int> ln = new List<int>(_left);
                ln.Add(0);
                ln.AddRange(_right);
                ln.Sort(this);
                return ln.ToArray();
            }
            set
            {
                if (value != null)
                {
                    List<int> ln = new List<int>(value);
                    int p0 = ln.IndexOf(0);
                    _left.Clear();
                    for (int ix = 0; ix < p0; ix++)
                    {
                        _left.Add(ln[ix]);
                    }
                    _right.Clear();
                    for (int ix = p0 + 1; ix < ln.Count; ix++)
                    {
                        _right.Add(ln[ix]);
                    }
                }
            }
        }
        private void SetHScroll()
        {
            int steps = 10 + (_length.Width - ((Width  - vScroll.Width) / sqs));
            if (steps > 0)
            {
                hScroll.Visible = true;
                hScroll.Value = 0;
                hScroll.Maximum = steps;
                hScroll.Value = steps / 2;
                hScroll.LargeChange = Math.Max(1, (steps * 10) / 100);
            }
            else
            {
                hScroll.Visible = false;
            }
        }
        private void SetVScroll()
        {
            int steps = 10 + (_length.Height - ((Height - hScroll.Height) / sqs));
            if (steps > 0)
            {
                vScroll.Visible = true;
                vScroll.Value = 0;
                vScroll.Maximum = steps;
                vScroll.Value = steps / 2;
                vScroll.LargeChange = Math.Max(1, (steps * 10) / 100);
            }
            else
            {
                vScroll.Visible = false;
            }
        }
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            int dx = hScroll.Value - (hScroll.Maximum / 2);
            int dy = vScroll.Value - (vScroll.Maximum / 2);
            int x0 = ((Width - sqs) / 2) - (dx * sqs);
            int y0 = ((Height - sqs) / 2) - (dy * sqs);
            int cntr = (_length.Width - 1) / 2;
            int cntl = (_length.Width - 1) - cntr;
            int cntd = (_length.Height - 1) / 2;
            int cntu = (_length.Height - 1) - cntd;
            Font f = new Font("Arial", 12f, GraphicsUnit.Pixel);
            float hfont = e.Graphics.MeasureString("0", f).Height;

            int n = 0;
            for (int iy = 0; iy < cntu; iy++)
            {
                for (int ix = 0; ix < _length.Width; ix++)
                {
                    int xp = x0 - sqs * (cntl - ix);
                    int yp = y0 - sqs * (iy + 1);
                    if ((xp < Width - vScroll.Width) && (yp < Height - hScroll.Height) && ((sqs + xp) >= 0) && ((sqs + yp) >= 0))
                    {
                        e.Graphics.DrawRectangle(Pens.Black, new Rectangle(xp, yp, sqs, sqs));
                        if (_left.Contains(-((cntl - ix) + ((iy + 1) * _length.Width))))
                        {
                            e.Graphics.FillRectangle(Brushes.Yellow, new Rectangle(1 + xp, yp + 1, sqs - 1, sqs - 1));
                            float ixw = e.Graphics.MeasureString(n.ToString(), f).Width;
                            e.Graphics.DrawString(n.ToString(), f, Brushes.Black, xp + (sqs - ixw) / 2f, yp + (sqs - hfont) / 2f);
                            n++;
                        }
                        else
                        {
                            e.Graphics.FillRectangle(Brushes.DarkGray, new Rectangle(1 + xp, yp + 1, sqs - 1, sqs - 1));
                        }
                    }
                }
            }
            e.Graphics.FillRectangle(Brushes.Black, new Rectangle(x0, y0, sqs + 1, sqs + 1));            
            for (int ix = 0; ix < cntl; ix++)
            {
                int xp = x0 - sqs * (ix + 1);
                if ((xp < Width - vScroll.Width) && (y0 < Height - hScroll.Height) && (xp + sqs >= 0) && (y0 + sqs >= 0))
                {
                    e.Graphics.DrawRectangle(Pens.Black, new Rectangle(xp, y0, sqs, sqs));
                    if (_left.Contains(-(ix + 1)))
                    {
                        e.Graphics.FillRectangle(Brushes.Yellow, new Rectangle(1 + xp, y0 + 1, sqs - 1, sqs - 1));
                        float ixw = e.Graphics.MeasureString(n.ToString(), f).Width;
                        e.Graphics.DrawString(n.ToString(), f, Brushes.Black, xp + (sqs - ixw) / 2f, y0 + (sqs - hfont) / 2f);
                        n++;
                    }
                    else
                    {
                        e.Graphics.FillRectangle(Brushes.DarkGray, new Rectangle(1 + xp, y0 + 1, sqs - 1, sqs - 1));
                    }
                }
            }
            for (int ix = 0; ix < cntr; ix++)
            {
                int xp = x0 + sqs * (ix + 1);
                if ((xp < Width - vScroll.Width) && (y0 < Height - hScroll.Height) && (xp + sqs >= 0) && (y0 + sqs >= 0))
                {
                    e.Graphics.DrawRectangle(Pens.Black, new Rectangle(xp, y0, sqs, sqs));
                    if (_right.Contains(ix + 1))
                    {
                        e.Graphics.FillRectangle(Brushes.Yellow, new Rectangle(1 + xp, y0 + 1, sqs - 1, sqs - 1));
                        float ixw = e.Graphics.MeasureString(n.ToString(), f).Width;
                        e.Graphics.DrawString(n.ToString(), f, Brushes.Black, xp + (sqs - ixw) / 2f, y0 + (sqs - hfont) / 2f);
                        n++;
                    }
                    else
                    {
                        e.Graphics.FillRectangle(Brushes.DarkGray, new Rectangle(1 + xp, y0 + 1, sqs - 1, sqs - 1));
                    }
                }
            }
            for (int iy = 0; iy < cntd; iy++)
            {
                for (int ix = 0; ix < _length.Width; ix++)
                {
                    int xp = x0 - sqs * (cntl - ix);
                    int yp = y0 + sqs * (iy + 1);
                    if ((xp < Width - vScroll.Width) && (yp < Height - hScroll.Height) && ((sqs + xp) >= 0) && ((sqs + yp) >= 0))
                    {
                        e.Graphics.DrawRectangle(Pens.Black, new Rectangle(xp, yp, sqs, sqs));
                        if (_right.Contains((ix - cntl) + ((iy + 1) * _length.Width)))
                        {
                            e.Graphics.FillRectangle(Brushes.Yellow, new Rectangle(1 + xp, yp + 1, sqs - 1, sqs - 1));
                            float ixw = e.Graphics.MeasureString(n.ToString(), f).Width;
                            e.Graphics.DrawString(n.ToString(), f, Brushes.Black, xp + (sqs - ixw) / 2f, yp + (sqs - hfont) / 2f);
                            n++;
                        }
                        else
                        {
                            e.Graphics.FillRectangle(Brushes.DarkGray, new Rectangle(1 + xp, yp + 1, sqs - 1, sqs - 1));
                        }
                    }
                }
            }
            f.Dispose();
        }
        protected override void OnSizeChanged(EventArgs e)
        {
            base.OnSizeChanged(e);
            SetHScroll();
            SetVScroll();
            Refresh();
        }
        protected override void OnMouseClick(MouseEventArgs e)
        {
            base.OnMouseClick(e);
            int dx = hScroll.Value - (hScroll.Maximum / 2);
            int dy = vScroll.Value - (vScroll.Maximum / 2);
            int x0 = ((Width - sqs) / 2) - (dx * sqs);
            int y0 = ((Height - sqs) / 2) - (dy * sqs);
            int xl = (int)Math.Ceiling((double)((e.X / sqs) - x0));
            int yl = (int)Math.Ceiling((double)((e.Y / sqs) - y0));
            if (e.X < x0)
            {
                xl = -((int)Math.Ceiling((double)(x0 - e.X) / sqs));
            }
            else
            {
                xl = (int)Math.Ceiling((double)(e.X - (x0 + sqs)) / sqs);
            }
            if (e.Y < y0)
            {
                yl = -((int)Math.Ceiling((double)(y0 - e.Y) / sqs));
            }
            else
            {
                yl = (int)Math.Ceiling((double)(e.Y - (y0 + sqs)) / sqs);
            }
            if ((xl == 0) && (yl == 0))
            {
                return;
            }
            xl = xl + (yl * _length.Width);
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                if (xl < 0)
                {
                    if (!_left.Contains(xl))
                    {
                        _left.Add(xl);
                        OnChangeNeighborhood?.Invoke(this, EventArgs.Empty);
                    }
                }
                else
                {
                    if (!_right.Contains(xl))
                    {
                        _right.Add(xl);
                        OnChangeNeighborhood?.Invoke(this, EventArgs.Empty);
                    }
                }
            }
            else if ((e.Button & MouseButtons.Right) == MouseButtons.Right)
            {
                if (xl < 0)
                {
                    _left.Remove(xl);
                    OnChangeNeighborhood?.Invoke(this, EventArgs.Empty);
                }
                else
                {
                    _right.Remove(xl);
                    OnChangeNeighborhood?.Invoke(this, EventArgs.Empty);
                }
            }
            Refresh();
        }

        private void hScroll_ValueChanged(object sender, EventArgs e)
        {
            Refresh();
        }

        private void vScroll_ValueChanged(object sender, EventArgs e)
        {
            Refresh();
        }
    }
}
