﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using CommonObjects.Interfaces;
using CellularAutomata.Properties;
using CellularAutomata.Interfaces;

namespace CellularAutomata.Controls
{
    public partial class GridEditor : Form
    {
        private ICAStateProvider _stProvider = null;
        private int _zoom = 1;
        private PictureBox _imgSel = null;
        private Bitmap _image = null;

        public GridEditor()
        {
            InitializeComponent();
            Text = UIResources.TTL_EDITOR;
            bZoomIn.ToolTipText = UIResources.TTP_ZOOMIN;
            bZoomOut.ToolTipText = UIResources.TTP_ZOOMOUT;
            bPatterns.ToolTipText = UIResources.TTP_PATTERNS;
            bPatternMode.ToolTipText = UIResources.TTP_PATMODE;
            bPixelMode.ToolTipText = UIResources.TTP_PIXMODE;
            bRandom.ToolTipText = UIResources.TTP_RANDOM;
            bFill.ToolTipText = UIResources.TTP_FILL;
            lState.Text = UIResources.LAB_STATE;
            bOK.ToolTipText = UIResources.TTP_OK;
            bCancel.ToolTipText = UIResources.TTP_CANCEL;
            ofDlg.Filter = UIResources.FIL_IMAGES;
        }
        public ICAStateProvider StateProvider
        {
            get
            {
                return _stProvider;
            }
            set
            {
                _stProvider = value;
                cbStates.Items.Clear();
                foreach (ICAState st in _stProvider.States)
                {
                    cbStates.Items.Add(st);
                }
                cbStates.SelectedIndex = 0;
            }
        }
        public Size ItemSize
        {
            get
            {
                return pbSample.Size;
            }
            set
            {
                pbSample.Size = new Size(value.Width * _zoom, value.Height * _zoom);
                _image = new Bitmap(value.Width, value.Height);
                SetSamplePosition();
            }
        }
        public void SetImageRow(int row, int[] states)
        {
            for (int ix = 0; ix < states.Length; ix++)
            {
                _image.SetPixel(ix, row, _stProvider.GetState(states[ix]).StateColor);
            }
        }
        public override void Refresh()
        {
            base.Refresh();
            SetImage();
        }
        public int[] GetImageRow(int row)
        {
            int[] data = new int[_image.Width];
            for (int ix = 0; ix < _image.Width; ix++)
            {
                data[ix] = GetState(_image.GetPixel(ix, row));
            }
            return data;
        }
        private void SetSamplePosition()
        {
            pbSample.Left = (scSample.Panel1.Width - pbSample.Width) / 2;
            pbSample.Top = (scSample.Panel1.Height - pbSample.Height) / 2;
        }
        private int GetState(Color c)
        {
            for (int s = 0; s < _stProvider.Count; s++)
            {
                if (c == _stProvider.GetState(s).StateColor)
                {
                    return s;
                }
            }
            return 0;
        }
        private void LoadPattern(string file)
        {
            try
            {
                PictureBox pb = new PictureBox();
                pb.SizeMode = PictureBoxSizeMode.CenterImage;
                Bitmap img = new Bitmap(file, false);
                for (int iy = 0; iy < img.Height; iy++)
                {
                    for (int ix = 0; ix < img.Width; ix++)
                    {
                        if (img.GetPixel(ix, iy).ToArgb() != Color.Black.ToArgb())
                        {
                            img.SetPixel(ix, iy, Color.White);
                        }
                    }
                }
                pb.Size = new Size(img.Width + 4, img.Height + 4);
                pb.Image = img;
                pb.Click += new EventHandler(PatternSelected);
                flpPatterns.Controls.Add(pb);
            }
            catch (Exception ex)
            {
                MessageBox.Show(file + ": " + ex.Message);
            }
        }

        private void PatternSelected(object sender, EventArgs e)
        {
            foreach (Control c in flpPatterns.Controls)
            {
                if (c != sender)
                {
                    c.BackColor = Color.Transparent;
                }
                else
                {
                    c.BackColor = Color.Red;
                    _imgSel = sender as PictureBox;
                }
            }
        }
        private void SetImage()
        {
            Bitmap bmp = new Bitmap(_image.Width * _zoom, _image.Height * _zoom);
            Graphics g = Graphics.FromImage(bmp);
            for (int iy = 0; iy < _image.Height; iy++)
            {
                for (int ix = 0; ix < _image.Width; ix++)
                {
                    using (SolidBrush br = new SolidBrush(_image.GetPixel(ix, iy)))
                    {
                        g.FillRectangle(br, new Rectangle(ix * _zoom, iy * _zoom, _zoom, _zoom));
                    }
                }
            }
            Bitmap bold = pbSample.Image as Bitmap;
            pbSample.Size = new Size(_image.Width * _zoom, _image.Height * _zoom);
            pbSample.Image = bmp;
            SetSamplePosition();
            if (bold != null)
            {
                bold.Dispose();
            }
        }

        private void bPixelMode_Click(object sender, EventArgs e)
        {
            bPixelMode.Checked = true;
            bPatternMode.Checked = false;
        }

        private void bPatternMode_Click(object sender, EventArgs e)
        {
            bPixelMode.Checked = false;
            bPatternMode.Checked = true;
        }

        private void bPatterns_Click(object sender, EventArgs e)
        {            
            if (ofDlg.ShowDialog() == DialogResult.OK)
            {
                foreach (string file in ofDlg.FileNames)
                {
                    LoadPattern(file);
                }
            }
        }

        private void bFill_Click(object sender, EventArgs e)
        {
            try
            {
                ICAState s = cbStates.SelectedItem as ICAState;
                if (s != null)
                {
                    Color c = s.StateColor;
                    Graphics g = Graphics.FromImage(_image);
                    using (SolidBrush br = new SolidBrush(c))
                    {
                        g.FillRectangle(br, new Rectangle(0, 0, _image.Width, _image.Height));
                    }
                    SetImage();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void scSample_Panel1_SizeChanged(object sender, EventArgs e)
        {
            SetSamplePosition();
        }

        private void bZoomOut_Click(object sender, EventArgs e)
        {
            if (_zoom > 1)
            {
                _zoom >>= 1;
                SetImage();
                SetSamplePosition();                
            }
        }

        private void bZoomIn_Click(object sender, EventArgs e)
        {
            if (_zoom < 32)
            {
                _zoom <<= 1;
                SetImage();
                SetSamplePosition();
            }
        }

        private void bRandom_Click(object sender, EventArgs e)
        {
            int np = 0;
            if (bPatternMode.Checked && (_imgSel == null))
            {
                MessageBox.Show(UIResources.ERR_MOPATERN);
                return;
            }
            if (int.TryParse(txtRandNum.Text, out np) && (np > 0))
            {
                Random r = new Random();
                Color c = (cbStates.SelectedItem as ICAState).StateColor;
                for (int cn = 0; cn < np; cn++)
                {
                    int x = r.Next(_image.Width);
                    int y = r.Next(_image.Height);
                    if (bPatternMode.Checked)
                    {
                        Bitmap bmp = _imgSel.Image as Bitmap;
                        for (int iy = y; iy < Math.Min(y + bmp.Height, _image.Height); iy++)
                        {
                            for (int ix = x; ix < Math.Min(x + bmp.Width, _image.Width); ix++)
                            {
                                if (bmp.GetPixel(ix - x, iy - y).ToArgb() != Color.Black.ToArgb())
                                {
                                    _image.SetPixel(ix, iy, c);
                                }
                            }
                        }
                    }
                    else
                    {
                        _image.SetPixel(x, y, c);
                    }
                }
                SetImage();
            }
            else
            {
                MessageBox.Show(UIResources.ERR_GABNUM);
                txtRandNum.SelectAll();
                txtRandNum.Focus();
            }
        }

        private void pbSample_MouseClick(object sender, MouseEventArgs e)
        {
            try
            {
                if (bPatternMode.Checked && (_imgSel == null))
                {
                    MessageBox.Show(UIResources.ERR_MOPATERN);
                    return;
                }
                int x = e.X / _zoom;
                int y = e.Y / _zoom;
                Color c = (cbStates.SelectedItem as ICAState).StateColor;
                if (bPatternMode.Checked)
                {
                    Bitmap bmp = _imgSel.Image as Bitmap;
                    for (int iy = y; iy < Math.Min(y + bmp.Height, _image.Height); iy++)
                    {
                        for (int ix = x; ix < Math.Min(x + bmp.Width, _image.Width); ix++)
                        {
                            if (bmp.GetPixel(ix - x, iy - y).ToArgb() != Color.Black.ToArgb())
                            {
                                _image.SetPixel(ix, iy, c);
                            }
                        }
                    }
                }
                else
                {
                    _image.SetPixel(x, y, c);
                }
                SetImage();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void bOK_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
        }

        private void bCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }

        private void GridEditor_Load(object sender, EventArgs e)
        {
            Refresh();
        }
    }
}
