﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CellularAutomata.Controls
{
    public partial class Linear1DCAEditor : UserControl
    {
        private const int sqs = 20;

        private List<int> _left = new List<int>();
        private List<int> _right = new List<int>();
        private int _length = 0;
        public Linear1DCAEditor()
        {
            InitializeComponent();
        }
        public event EventHandler OnChangeNeighborhood = null;
        public int Length
        {
            get
            {
                return _length;
            }
            set
            {
                _length = value;
                if (_length > 0)
                {
                    int cntr = (_length - 1) / 2;
                    int cntl = (_length - 1) - cntr;
                    for (int ix = _left.Count - 1; ix >= 0; ix--)
                    {
                        if (_left[ix] < -cntl)
                        {
                            _left.RemoveAt(ix);
                        }
                    }
                    for (int ix = _right.Count - 1; ix >= 0; ix--)
                    {
                        if (_right[ix] > cntr)
                        {
                            _right.RemoveAt(ix);
                        }
                    }
                    SetScroll();
                    Refresh();
                }
            }
        }
        public int[] Neighborhood
        {
            get
            {
                _left.Sort();
                _left.Reverse();
                _right.Sort();
                List<int> ln = new List<int>(_left);
                ln.Add(0);
                ln.AddRange(_right);
                return ln.ToArray();
            }
            set
            {
                if (value != null)
                {
                    List<int> ln = new List<int>(value);
                    int p0 = ln.IndexOf(0);
                    _left.Clear();
                    for (int ix = 0; ix < p0; ix++)
                    {
                        _left.Add(ln[ix]);
                    }
                    _right.Clear();
                    for (int ix = p0 + 1; ix < ln.Count; ix++)
                    {
                        _right.Add(ln[ix]);
                    }
                }
            } 
        }
        private void SetScroll()
        {
            int steps = 10 + (_length - (Width / sqs));
            if (steps > 0)
            {
                hScroll.Visible = true;
                hScroll.Value = 0;
                hScroll.Maximum = steps;
                hScroll.Value = steps / 2;
                hScroll.LargeChange = Math.Max(1, (steps * 10) / 100);
            }
            else
            {
                hScroll.Visible = false;
            }
        }
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            int dx = hScroll.Value - (hScroll.Maximum / 2);
            int x0 = ((Width - sqs) / 2) - (dx * sqs);
            int y0 = (Height - sqs) / 2;
            int cntr = (_length - 1) / 2;
            int cntl = (_length - 1) - cntr;
            Font f = new Font("Arial", 12f, GraphicsUnit.Pixel);
            float hfont = e.Graphics.MeasureString("0", f).Height;
            e.Graphics.FillRectangle(Brushes.Black, new Rectangle(x0, y0, sqs + 1, sqs + 1));
            int n = 0;
            for (int ix = 0; ix < cntl; ix++)
            {
                e.Graphics.DrawRectangle(Pens.Black, new Rectangle(x0 - sqs * (ix + 1), y0, sqs, sqs));
                if (_left.Contains(-(ix + 1)))
                {
                    e.Graphics.FillRectangle(Brushes.Yellow, new Rectangle(1 + (x0 - sqs * (ix + 1)), y0 + 1, sqs - 1, sqs - 1));
                    float ixw = e.Graphics.MeasureString(n.ToString(), f).Width;
                    e.Graphics.DrawString(n.ToString(), f, Brushes.Black, (x0 - sqs * (ix + 1)) + (sqs - ixw) / 2f, y0 + (sqs - hfont) / 2f);
                    n++;
                }
                else
                {
                    e.Graphics.FillRectangle(Brushes.DarkGray, new Rectangle(1 + (x0 - sqs * (ix + 1)), y0 + 1, sqs - 1, sqs - 1));
                }
                float txtw = e.Graphics.MeasureString((-(ix + 1)).ToString(), f).Width;
                e.Graphics.DrawString((-(ix + 1)).ToString(), f, Brushes.Black, (x0 - sqs * (ix + 1)) + (sqs - txtw) / 2f, y0 - (hfont + 4f));
            }
            for (int ix = 0; ix < cntr; ix++)
            {
                e.Graphics.DrawRectangle(Pens.Black, new Rectangle(x0 + sqs * (ix + 1), y0, sqs, sqs));
                if (_right.Contains(ix + 1))
                {
                    e.Graphics.FillRectangle(Brushes.Yellow, new Rectangle(1 + (x0 + sqs * (ix + 1)), y0 + 1, sqs - 1, sqs - 1));
                    float ixw = e.Graphics.MeasureString(n.ToString(), f).Width;
                    e.Graphics.DrawString(n.ToString(), f, Brushes.Black, (x0 + sqs * (ix + 1)) + (sqs - ixw) / 2f, y0 + (sqs - hfont) / 2f);
                    n++;
                }
                else
                {
                    e.Graphics.FillRectangle(Brushes.DarkGray, new Rectangle(1 + (x0 + sqs * (ix + 1)), y0 + 1, sqs - 1, sqs - 1));
                }
                float txtw = e.Graphics.MeasureString((ix + 1).ToString(), f).Width;
                e.Graphics.DrawString((ix + 1).ToString(), f, Brushes.Black, (x0 + sqs * (ix + 1)) + (sqs - txtw) / 2f, y0 - (hfont + 4f));
            }
            f.Dispose();
        }
        protected override void OnSizeChanged(EventArgs e)
        {
            base.OnSizeChanged(e);
            SetScroll();
            Refresh();
        }
        protected override void OnMouseClick(MouseEventArgs e)
        {
            base.OnMouseClick(e);
            int dx = hScroll.Value - (hScroll.Maximum / 2);
            int x0 = ((Width - sqs) / 2) - (dx * sqs);
            int y0 = (Height - sqs) / 2;
            if ((e.Y >= y0) && (e.Y <= y0 + sqs))
            {
                int xl = 0;
                if (e.X < x0)
                {
                    xl = -((int)Math.Ceiling((double)(x0 - e.X) / sqs));
                }
                else if (e.X >= x0 + sqs)
                {
                    xl = (int)Math.Ceiling((double)(e.X - (x0 + sqs)) / sqs);
                }
                else
                {
                    return;
                }
                if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
                {
                    if (xl < 0)
                    {
                        if (!_left.Contains(xl))
                        {
                            _left.Add(xl);
                            OnChangeNeighborhood?.Invoke(this, EventArgs.Empty);
                        }
                    }
                    else
                    {
                        if (!_right.Contains(xl))
                        {
                            _right.Add(xl);
                            OnChangeNeighborhood?.Invoke(this, EventArgs.Empty);
                        }
                    }
                }
                else if ((e.Button & MouseButtons.Right) == MouseButtons.Right)
                {
                    if (xl < 0)
                    {
                        _left.Remove(xl);
                        OnChangeNeighborhood?.Invoke(this, EventArgs.Empty);
                    }
                    else
                    {
                        _right.Remove(xl);
                        OnChangeNeighborhood?.Invoke(this, EventArgs.Empty);
                    }
                }
                Refresh();
            }
        }

        private void hScroll_ValueChanged(object sender, EventArgs e)
        {
            Refresh();
        }
    }
}
