﻿using System;
using System.Drawing;
using System.ComponentModel;
using System.Text.RegularExpressions;
using CommonObjects.Interfaces;
using CellularAutomata.Utils;
using CellularAutomata.Properties;

namespace CellularAutomata.Data
{
    [Serializable]
    public abstract class PropertyValue
    {
        private string _name;
        public PropertyValue()
        {
        }
        public PropertyValue(IObjectProperty p)
        {
            Name = p.Name;
            VarValue = p.Value.Clone();
        }
        public static PropertyValue FromProperty(IObjectProperty p)
        {
            if (p.Value.NativeType == typeof(int))
            {
                return new IntPropertyValue(p);
            }
            if (p.Value.NativeType == typeof(double))
            {
                return new DoublePropertyValue(p);
            }
            if (p.Value.NativeType == typeof(bool))
            {
                return new BoolPropertyValue(p);
            }
            if (p.Value.NativeType == typeof(string))
            {
                return new StringPropertyValue(p);
            }
            if (p.Value.NativeType == typeof(Color))
            {
                return new ColorPropertyValue(p);
            }
            return null;
        }
        [CategoryGlobal("CAT_PROPERTY")]
        [DisplayNameGlobal("DIS_NAME")]
        [DescriptionGlobal("DES_NAME")]
        public string Name
        {
            get
            {
                return _name;
            }
            private set
            {
                Regex reg = new Regex("^[a-zA-Z][a-zA-Z0-9]*$");
                if (reg.IsMatch(value))
                {
                    _name = value;
                }
                else
                {
                    throw new Exception(UIResources.ERR_NAME);
                }
            }
        }
        [Browsable(false)]
        public IVariantValue VarValue { get; set; }
        public void ChangeName(string name)
        {
            Name = name;
        }
        public override string ToString()
        {
            return Name;
        }
    }
    [Serializable]
    public class IntPropertyValue : PropertyValue
    {
        public IntPropertyValue() : base()
        {
        }
        public IntPropertyValue(IObjectProperty p) : base(p)
        {
        }
        [CategoryGlobal("CAT_DATA")]
        [DisplayNameGlobal("DIS_VALUE")]
        [DescriptionGlobal("DES_VALUE")]
        public int Value
        {
            get
            {
                return VarValue.AsInt;
            }
            set
            {
                VarValue = VarValue.FromObject(value);
            }
        }
    }
    [Serializable]
    public class DoublePropertyValue : PropertyValue
    {
        public DoublePropertyValue() : base()
        {
        }
        public DoublePropertyValue(IObjectProperty p) : base(p)
        {
        }
        [CategoryGlobal("CAT_DATA")]
        [DisplayNameGlobal("DIS_VALUE")]
        [DescriptionGlobal("DES_VALUE")]
        public double Value
        {
            get
            {
                return VarValue.AsDouble;
            }
            set
            {
                VarValue = VarValue.FromObject(value);
            }
        }
    }
    [Serializable]
    public class BoolPropertyValue : PropertyValue
    {
        public BoolPropertyValue() : base()
        {
        }
        public BoolPropertyValue(IObjectProperty p) : base(p)
        {
        }
        [CategoryGlobal("CAT_DATA")]
        [DisplayNameGlobal("DIS_VALUE")]
        [DescriptionGlobal("DES_VALUE")]
        public bool Value
        {
            get
            {
                return VarValue.AsBool;
            }
            set
            {
                VarValue = VarValue.FromObject(value);
            }
        }
    }
    [Serializable]
    public class StringPropertyValue : PropertyValue
    {
        public StringPropertyValue() : base()
        {
        }
        public StringPropertyValue(IObjectProperty p) : base(p)
        {
        }
        [CategoryGlobal("CAT_DATA")]
        [DisplayNameGlobal("DIS_VALUE")]
        [DescriptionGlobal("DES_VALUE")]
        public string Value
        {
            get
            {
                return VarValue.AsString;
            }
            set
            {
                VarValue = VarValue.FromObject(value);
            }
        }
    }
    [Serializable]
    public class ColorPropertyValue : PropertyValue
    {
        public ColorPropertyValue() : base()
        {
        }
        public ColorPropertyValue(IObjectProperty p) : base(p)
        {
        }
        [CategoryGlobal("CAT_DATA")]
        [DisplayNameGlobal("DIS_VALUE")]
        [DescriptionGlobal("DES_VALUE")]
        public Color Value
        {
            get
            {
                return VarValue.AsColor;
            }
            set
            {
                VarValue = VarValue.FromObject(value);
            }
        }
    }
}
