﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using BNFUP.Interfaces;
using BNFUP.Rules;
using CommonObjects.Expressions;
using CommonObjects.Interfaces;
using CellularAutomata.Properties;

namespace CellularAutomata.Expressions
{
    public class ExpressionFactory : ICompilableObjectFactory
    {
        private LogicExpression _root = null;

        public ExpressionFactory()
        {
        }
        public IValueProvider ValueProvider { get; set; }
        public RuleTable Compiler
        {
            get
            {
                MemoryStream ms = new MemoryStream(Resources.expressions);
                RuleTable rt = RuleTable.FromStream(ms);
                rt.Factory = this;
                return rt;
            }
        }
        #region ICompilableObjectFactory
        public ICompilableObject Object
        {
            get
            {
                ICompilableObject obj = _root.Simplify();
                return obj;
            }
        }
        public void Init()
        {
            _root = null;
        }
        public ICompilableObject CreateObject(string ctype, IRuleItem item)
        {
            switch (ctype)
            {
                case "number":
                    return new Number();
                case "string":
                    return new StringLiteral();
                case "booloperator":
                    return new BoolOperator();
                case "reloperator":
                    return new RelOperator();
                case "arithperator":
                    return new ArithmeticOperator();
                case "typedef":
                    return new TypeDef();
                case "truthfunc":
                    return new TruthFunction() { ValueProvider = ValueProvider };
                case "videntifier":
                    return new VarIdentifier();
                case "pidentifier":
                    return new PropIdentifier();
                case "valref":
                    return new VariableReference() { ValueProvider = ValueProvider };
                case "propref":
                    return new PropertyReference() { ValueProvider = ValueProvider };
                case "indxpropref":
                    return new IndexedProperty() { ValueProvider = ValueProvider };
                case "iteratorpropref":
                    return new IteratorProperty() { ValueProvider = ValueProvider };
                case "strbfunction":
                    return new StrBoolFunction() { ValueProvider = ValueProvider };
                case "boolagg":
                    return new BoolAggregate() { ValueProvider = ValueProvider };
                case "functionname":
                    return new FunctionName();
                case "relexpr":
                    return new RelExpression() { ValueProvider = ValueProvider };
                case "arithexpr":
                    return new ArithmeticExpression() { ValueProvider = ValueProvider };
                case "parithexpr":
                    return new ArithmeticExpression() { Parenthesis = true, ValueProvider = ValueProvider };
                case "kvalue":
                    return new ArithmeticValue() { ValueProvider = ValueProvider };
                case "kbool":
                    return new BoolValue();
                case "intfunction":
                    return new IntFunction() { ValueProvider = ValueProvider };
                case "icount":
                    return new IntCountAggregate() { ValueProvider = ValueProvider };
                case "intagg":
                    return new IntAggregateFunction() { ValueProvider = ValueProvider };
                case "dfunction":
                    return new DoubleFunction() { ValueProvider = ValueProvider };
                case "dcount":
                    return new DoubleCountAggregate() { ValueProvider = ValueProvider };
                case "doubleagg":
                    return new DoubleAggregateFunction() { ValueProvider = ValueProvider };
                case "intstrf":
                    return new StrIntFunction() { ValueProvider = ValueProvider };
                case "strsubs":
                    return new StrSubstringFunction() { ValueProvider = ValueProvider };
                case "strstr":
                    return new StrConcatFunction() { ValueProvider = ValueProvider };
                case "strrepl":
                    return new StrReplaceFunction() { ValueProvider = ValueProvider };
                case "boolexpr":
                    LogicExpression ex = new LogicExpression() { ValueProvider = ValueProvider };
                    if (_root == null)
                    {
                        _root = ex;
                    }
                    return ex;
                case "pboolexpr":
                    LogicExpression pex = new LogicExpression() { ValueProvider = ValueProvider };
                    pex.Parenthesis = true;
                    if (_root == null)
                    {
                        _root = pex;
                    }
                    return pex;
                default:
                    return null;
            }
        }
        #endregion
    }
}
