﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using CommonObjects.Expressions;
using CommonObjects.Interfaces;
using CellularAutomata.Interfaces;
using CellularAutomata.Properties;

namespace CellularAutomata.Expressions
{
    public partial class frmTest : Form, IValueProvider
    {
        private ExprBase _expression = null;
        private ICAStateProvider _stprovider = null;
        private int _stIndex = 0;
        public frmTest()
        {
            InitializeComponent();
            lCurrent.Text = UIResources.LAB_CURRSTATE;
            bClearNb.ToolTipText = UIResources.TTP_CLEARNB;
        }
        public ExprBase Expression
        {
            get
            {
                return _expression;
            }
            set
            {
                _expression = value;
                _expression.ValueProvider = this;         
                CreateTree();
            }
        }
        public IPropertyProvider VariableProvider { get; set; }
        public ICAStateProvider StateProvider
        {
            get
            {
                return _stprovider;
            }
            set
            {
                _stprovider = value;
                ConfigureToolbars();
            }
        }
        #region IValueProvider
        public IVariantValue GetVarValue(string name)
        {
            return VariableProvider.GetProperty(name).Value;
        }
        public IVariantValue GetPropertyValue(string name)
        {
            if (cbCurrState.SelectedItem != null)
            {
                return StateProvider.GetState((cbCurrState.SelectedItem as ICAState).Name).GetVarValue(name);
            }
            return null;
        }
        public IVariantValue GetIteratorPropertyValue(string name)
        {
            return GetIndexedPropertyValue(_stIndex, name);
        }
        public IVariantValue GetIndexedPropertyValue(int ix, string name)
        {
            int cc = 0;
            foreach (ToolStripItem it in tsNeighbours.Items)
            {
                if (it is ToolStripComboBox)
                {
                    if (cc == ix) 
                    {
                        if ((it as ToolStripComboBox).SelectedItem == null)
                        {
                            return null;
                        }
                        return StateProvider.GetState(((it as ToolStripComboBox).SelectedItem as ICAState).Name).GetVarValue(name);
                    }
                    else
                    {
                        cc++;
                    }
                }
            }
            return null;
        }
        public void BeginIterator()
        {
            _stIndex = 0;
        }
        public bool Next()
        {
            _stIndex++;
            return _stIndex < ((tsNeighbours.Items.Count / 3) - 1);
        }
        public void EndIterator()
        {
        }
        #endregion
        private void ConfigureToolbars()
        {
            cbCurrState.Items.Clear();
            if (_stprovider != null)
            {
                foreach (ICAState c in _stprovider.States)
                {
                    cbCurrState.Items.Add(c);
                }
            }
            ClearNeighbours();
        }
        private void ClearNeighbours()
        {
            while(tsNeighbours.Items.Count > 1)
            {
                tsNeighbours.Items.RemoveAt(1);
            }
            if (_stprovider != null)
            {
                ToolStripLabel lbl = new ToolStripLabel(string.Format(UIResources.LAB_NBSTATE, 0));
                tsNeighbours.Items.Insert(tsNeighbours.Items.Count - 1, lbl);
                ToolStripComboBox cb = new ToolStripComboBox();
                cb.DropDownStyle = ComboBoxStyle.DropDownList;
                foreach (ICAState c in _stprovider.States)
                {
                    cb.Items.Add(c);
                }
                cb.SelectedIndex = -1;
                cb.SelectedIndexChanged += new EventHandler(cbSelectedIndexChanged);
                tsNeighbours.Items.Insert(tsNeighbours.Items.Count - 1, cb);
                tsNeighbours.Items.Insert(tsNeighbours.Items.Count - 1, new ToolStripSeparator());
            }
        }
        private void AddChilds(ExprBase eb, TreeNodeCollection nodes)
        {
            TreeNode node = new TreeNode(eb.AsText);
            node.Tag = eb;
            nodes.Add(node);
            ExprBase ex = eb as ExprBase;
            if (ex != null)
            {
                for (int ix = 0; ix < ex.OperandCount; ix++)
                {
                    ExprBase e = ex.Operand(ix);
                    if (e != null)
                    {
                        AddChilds(e, node.Nodes);
                    }
                }
            }
        }
        private void CreateTree()
        {
            tvExpression.Nodes.Clear();
            if (_expression != null)
            {
                AddChilds(_expression, tvExpression.Nodes);                
            }
        }

        private void tvExpression_AfterSelect(object sender, TreeViewEventArgs e)
        {
            try
            {
                lExpression.Text = "";
                if (e.Node != null)
                {
                    ExprBase eb = e.Node.Tag as ExprBase;
                    if (eb != null)
                    {
                        switch (eb.PreferredType)
                        {
                            case ExprType.Bool:
                                lExpression.Text = eb.AsText + " = " + eb.AsBool.ToString();
                                break;
                            case ExprType.Double:
                                lExpression.Text = eb.AsText + " = " + eb.AsDouble.ToString();
                                break;
                            case ExprType.Int:
                                lExpression.Text = eb.AsText + " = " + eb.AsInt.ToString();
                                break;
                            case ExprType.String:
                                lExpression.Text = eb.AsText + " = " + eb.AsString;
                                break;
                            default:
                                lExpression.Text = eb.AsText;
                                break;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        private void cbSelectedIndexChanged(object sender, EventArgs e)
        {
            ToolStripComboBox cb = sender as ToolStripComboBox;
            if ((cb != null) && (cb.SelectedIndex >= 0))
            {
                if (tsNeighbours.Items.IndexOf(cb) == tsNeighbours.Items.Count - 3)
                {                    
                    ToolStripLabel lbl = new ToolStripLabel(string.Format(UIResources.LAB_NBSTATE, (tsNeighbours.Items.Count - 1) / 3));
                    tsNeighbours.Items.Insert(tsNeighbours.Items.Count - 1, lbl);
                    ToolStripComboBox ncb = new ToolStripComboBox();
                    ncb.DropDownStyle = ComboBoxStyle.DropDownList;
                    foreach (ICAState c in _stprovider.States)
                    {
                        ncb.Items.Add(c);
                    }
                    ncb.SelectedIndex = -1;
                    ncb.SelectedIndexChanged += new EventHandler(cbSelectedIndexChanged);
                    tsNeighbours.Items.Insert(tsNeighbours.Items.Count - 1, ncb);
                    tsNeighbours.Items.Insert(tsNeighbours.Items.Count - 1, new ToolStripSeparator());
                }
                tvExpression_AfterSelect(sender, new TreeViewEventArgs(tvExpression.SelectedNode));
            }
        }

        private void bClearNb_Click(object sender, EventArgs e)
        {
            if (tsNeighbours.Items.Count > 4)
            {
                tsNeighbours.Items.RemoveAt(tsNeighbours.Items.Count - 2);
                tsNeighbours.Items.RemoveAt(tsNeighbours.Items.Count - 2);
                tsNeighbours.Items.RemoveAt(tsNeighbours.Items.Count - 2);
                (tsNeighbours.Items[tsNeighbours.Items.Count - 3] as ToolStripComboBox).SelectedIndex = -1;
            }
        }

        private void cbCurrState_SelectedIndexChanged(object sender, EventArgs e)
        {
            tvExpression_AfterSelect(sender, new TreeViewEventArgs(tvExpression.SelectedNode));
        }
    }
}
