﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CellularAutomata.Interfaces
{
    /// <summary>
    /// Cellular automata cell
    /// </summary>
    public interface ICACell
    {
        /// <summary>
        /// Gets or sets the index of the actual cell state
        /// </summary>
        int ActualState { get; set; }
        /// <summary>
        /// Gets or sets the index of the next cell state
        /// </summary>
        int NextState { get; set; }
        /// <summary>
        /// Enumerate the indexes of all the neighbor cells
        /// </summary>
        IEnumerable<int> Neighborhood { get; }
        /// <summary>
        /// Gets the number of neighbors
        /// </summary>
        int NeighborCount { get; }
        /// <summary>
        /// Add the indexes of all the neighbor cells
        /// </summary>
        /// <param name="ixn">Array with the indexes of the neighbor cells</param>
        void AddNeighbours(int[] ixn);
        /// <summary>
        /// Gets the indexes of all the neighbor cells
        /// </summary>
        /// <returns>Array of integers with the neighbor cells indexes</returns>
        int[] GetNeighborhood();
        /// <summary>
        /// Get the cell index of the nth neighbor
        /// </summary>
        /// <param name="ix">Neighbor number</param>
        /// <returns>Index of the nth neighbor cell</returns>
        int GetNeighbour(int ix);
        /// <summary>
        /// Remove all the neighbor references
        /// </summary>
        void ClearNeighborhood();
        /// <summary>
        /// Get the actual state of the cell index
        /// </summary>
        /// <param name="step">Actual step of the automata</param>
        /// <returns>Index of the actual state of the cell</returns>
        int GetState(int step);
    }
}
