﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;
using System.Windows.Forms;
using CommonObjects.Interfaces;

namespace CellularAutomata.Interfaces
{
    /// <summary>
    /// Interface for the cellular automata itself
    /// </summary>
    public interface ICANetwork : IValueProvider
    {        
        /// <summary>
        /// Gets or sets an optimizer to speed up the automata evolution
        /// </summary>
        ICAOptimizer Optimizer { get; set; }
        /// <summary>
        /// Gets or sets the cell states provider
        /// </summary>
        ICAStateProvider StateProvider { get; set; }
        /// <summary>
        /// Gets or sets the variable values provider
        /// </summary>
        IPropertyProvider VariableProvider { get; set; }
        /// <summary>
        /// Gets or sets the cell provider
        /// </summary>
        ICACellProvider CellProvider { get; set; }        
        /// <summary>
        /// Gets the actual image of the automata in form of a Bitmap object
        /// </summary>
        Bitmap ActualImage { get; }
        /// <summary>
        /// Gets the control used internally to set up the automata neighborhood
        /// </summary>
        Control Editor { get; }
        /// <summary>
        /// Gets the actual step of the automata evolution
        /// </summary>
        int ActualStep { get; }
        /// <summary>
        /// Provide user interface controls to configure the automata
        /// </summary>
        /// <param name="tb">Tool Strip bar to dock the controls</param>
        /// <param name="ix">Index in the ToolStrip bar to locate the controls</param>
        /// <param name="remove">true to remove the controls, false to crete them</param>
        void ConfigureToolBar(ToolStrip tb, int ix, bool remove);
        /// <summary>
        /// Initialize automata internal state
        /// </summary>
        void Initialize();
        /// <summary>
        /// Return to the step 0
        /// </summary>
        /// <returns>Bitmap with the initial automata image</returns>
        Bitmap Reset();
        /// <summary>
        /// Perform a step
        /// </summary>
        /// <returns>Bitmap with the actual automata image</returns>
        Bitmap Step();
        /// <summary>
        /// Increase automata image zoom
        /// </summary>
        /// <returns>Bitmap with the new automata image</returns>
        Bitmap ZoomIn();
        /// <summary>
        /// Decrease automata image zoom
        /// </summary>
        /// <returns>Bitmap with the new automata image</returns>
        Bitmap ZoomOut();
    }
}
