﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CellularAutomata.Interfaces
{
    /// <summary>
    /// Cellular automata optimizer
    /// </summary>
    public interface ICAOptimizer
    {
        /// <summary>
        /// Gets or sets the optimizer maximum capacity
        /// </summary>
        int MaxCapacity { get; set; }
        /// <summary>
        /// Add a new transition, providing the initial and final state indexes, along with the neighborhood states
        /// </summary>
        /// <param name="from">Index of the initial state</param>
        /// <param name="neighborhood">Array with the state indexes of the neighbour cells</param>
        /// <param name="to">Index of the final state</param>
        void AddKnownTransition(int from, int[] neighborhood, int to);
        /// <summary>
        /// Retrieve the final state of a transition from the initial and neighborhood states
        /// </summary>
        /// <param name="from">Index of the initial state</param>
        /// <param name="neighborhood">Array with the state indexes of the neighbour cells</param>
        /// <returns>Index of the final state or -1 if unknown</returns>
        int GetTransition(int from, int[] neighborhood);
    }
}
