﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using CommonObjects.Properties;
using CommonObjects.Interfaces;

namespace CommonObjects.Data
{
    [Serializable]
    public class Constant : IVariantValue, IObjectProperty, IEquatable<IObjectProperty>, IComparable<IObjectProperty>
    {
        private IVariantValue _value;
        private Type _defaultValueType = typeof(IntVariantValue);
        public Constant()
        {
        }
        public Constant(object value)
        {
            _value = CreateValue(value);
        }
        public Constant(IVariantValue value)
        {
            _value = value;
        }
        #region IEquatable<IObjectProperty>
        public bool Equals(IObjectProperty other)
        {
            return string.Compare(other.Name, Name, true) == 0;
        }
        #endregion
        #region IComparable<IObjectProperty> 
        public int CompareTo(IObjectProperty other)
        {
            return string.Compare(Name, other.Name, true);
        }
        #endregion
        #region IObjectProperty
        public string Name { get; set; }
        public IVariantValue Value
        {
            get
            {
                return this;
            }
        }
        public Type DefaultValueType
        {
            get
            {
                return _defaultValueType;
            }
            set
            {
                if (value.GetInterface("IVariantValue") == null)
                {
                    throw new Exception(UIResources.ERR_IVARIANTVALUE);
                }
                _defaultValueType = value;
            }
        }
        public IVariantValue CreateValue(object value)
        {
            IVariantValue v = (IVariantValue)_defaultValueType.GetConstructor(Type.EmptyTypes).Invoke(null);
            return v.FromObject(value);
        }
        #endregion
        #region IVariantValue        
        public bool Mutable
        {
            get
            {
                return false;
            }
        }
        public int AsInt
        {
            get
            {
                return _value.AsInt;
            }
            set
            {
                throw new NotSupportedException(UIResources.ERR_CONSTVALUE);
            }
        }
        public double AsDouble
        {
            get
            {
                return _value.AsDouble;
            }
            set
            {
                throw new NotSupportedException(UIResources.ERR_CONSTVALUE);
            }
        }
        public Color AsColor
        {
            get
            {
                return _value.AsColor;
            }
            set
            {
                throw new NotSupportedException(UIResources.ERR_CONSTVALUE);
            }
        }
        public string AsString
        {
            get
            {
                return _value.AsString;
            }
            set
            {
                throw new NotSupportedException(UIResources.ERR_CONSTVALUE);
            }
        }
        public bool AsBool
        {
            get
            {
                return _value.AsBool;
            }
            set
            {
                throw new NotSupportedException(UIResources.ERR_CONSTVALUE);
            }
        }
        public object AsObject
        {
            get
            {
                return _value.AsObject;
            }
            set
            {
                _value.AsObject = value;
            }
        }
        public Type NativeType
        {
            get
            {
                return _value.NativeType;
            }
        }
        public IVariantValue FromObject(object value)
        {
            IVariantValue v = CreateValue(value);
            return new Constant(v);
        }
        public IVariantValue ConvertFrom(IVariantValue value)
        {
            return new Constant(_value.ConvertFrom(value));
        }
        public IVariantValue Clone()
        {
            return new Constant(_value.Clone());
        }
        #endregion
        public override string ToString()
        {
            return Name;
        }
    }
}
