﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class ArithmeticExpression : ArithmeticArgument
    {
        private ExprBase _exp1 = null;
        private ExprBase _exp2 = null;
        private ArithmeticOperator _op = null;

        public ArithmeticExpression()
        {
        }
        public override bool Indexed
        {
            get
            {
                if (_exp2 == null)
                {
                    return _exp1.Indexed;
                }
                return _exp1.Indexed || _exp2.Indexed;
            }
        }
        public override string AsText
        {
            get
            {
                if (_exp1 == null)
                {
                    return "ArithmeticExpression";
                }
                string paro = Parenthesis ? "(" : "";
                string parc = Parenthesis ? ")" : "";
                if (_exp2 == null)
                {
                    if (_op == null)
                    {
                        return  paro + _exp1.AsText + parc;
                    }
                    else
                    {
                        return paro + _op.Text + _exp1.AsText + parc;
                    }
                }
                else
                {
                    return paro + _exp1.AsText + _op.Text + _exp2.AsText + parc;
                }
            }
        }
        public override int OperandCount
        {
            get
            {
                return 2;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _exp1;
                case 1:
                    return _exp2;
                default:
                    return null;
            }
        }
        private void GroupLeft()
        {
            if ((_exp1 != null) && (_op != null) && (_exp2 != null) && (_exp2 is ArithmeticExpression))
            {
                ArithmeticExpression exp = _exp2 as ArithmeticExpression;
                if ((!exp.Parenthesis) && (exp._op != null) && (exp._exp2 != null) && (_op.Precedence == exp._op.Precedence))
                {
                    ArithmeticExpression exp1 = new ArithmeticExpression();
                    exp1.ValueProvider = _provider;
                    exp1._exp1 = _exp1;
                    exp1._op = _op;
                    exp1._exp2 = exp._exp1;
                    _exp1 = exp1;
                    _op = exp._op;
                    _exp2 = exp._exp2;
                }
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            bool accept = false;
            if ((_exp1 == null) && ((item is ArithmeticArgument) || (item is UniversalArgument)))
            {
                _exp1 = item as ExprBase;
                _exp1.ValueProvider = _provider;
                accept = true;
            }
            else if ((_op == null) && (item is ArithmeticOperator))
            {
                _op = item as ArithmeticOperator;
                accept = true;
            }
            else if ((_exp2 == null) && ((item is ArithmeticArgument) || (item is UniversalArgument)))
            {
                _exp2 = item as ExprBase;
                _exp2.ValueProvider = _provider;
                accept = true;
            }
            if (accept)
            {
                GroupLeft();
            }
            return accept;
        }
        public override ICompilableObject Simplify()
        {
            if (_exp1 != null)
            {
                _exp1 = _exp1.Simplify() as ExprBase;
                if ((_exp2 == null) && (_op == null))
                {
                    ArithmeticExpression ex1 = _exp1 as ArithmeticExpression;
                    if (ex1 != null)
                    {
                        _exp2 = ex1._exp2;
                        _op = ex1._op;
                        _exp1 = ex1._exp1;
                        Parenthesis = Parenthesis || ex1.Parenthesis;
                    }
                    else
                    {
                        return _exp1;
                    }
                }
                else if (_exp2 != null)
                {
                    _exp2 = _exp2.Simplify() as ExprBase;
                }
                else if ((_op != null) && (_op.Text == "-") && (_exp1 is Number))
                {
                    _exp1.Text = "-" + _exp1.Text;
                    return _exp1;
                }
                GroupLeft();
            }
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_exp1 != null)
                {
                    _exp1.ValueProvider = value;
                }
                if (_exp2 != null)
                {
                    _exp2.ValueProvider = value;
                }
            }
        }
        public override double AsDouble
        {
            get
            {
                if (_exp2 == null)
                {
                    if (_op != null)
                    {
                        return _op.Operate(_exp1.AsDouble);
                    }
                    return _exp1.AsDouble;
                }
                return _op.Operate(_exp1.AsDouble, _exp2.AsDouble);
            }
        }
        public override int AsInt
        {
            get
            {
                if (_exp2 == null)
                {
                    if (_op != null)
                    {
                        return _op.Operate(_exp1.AsInt);
                    }
                    return _exp1.AsInt;
                }
                return _op.Operate(_exp1.AsInt, _exp2.AsInt);
            }
        }
        public override Color AsColor
        {
            get
            {
                return Color.FromArgb(AsInt);
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                if (_exp2 == null)
                {
                    return _exp1.PreferredType;
                }
                if (_exp1.PreferredType != _exp2.PreferredType)
                {
                    return ExprType.Double;
                }
                return _exp1.PreferredType;
            }
        }
    }
}
