﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public abstract class CountAggregate : ArithmeticArgument
    {
        protected ExprBase _expr = null;
        public CountAggregate() : base()
        {
        }
        public override bool Indexed
        {
            get
            {
                return _expr.Indexed;
            }
        }
        public override string Text
        {
            get
            {
                return _text;
            }
            set
            {
            }
        }
        public override string AsText
        {
            get
            {
                if (_expr == null)
                {
                    return "CountAggregate: " + _text;
                }
                return _text + "(" + _expr.AsText + ")";
            }
        }
        public override int OperandCount
        {
            get
            {
                return 1;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _expr;
                default:
                    return null;
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            if (item is FunctionName)
            {
                _text = item.Text;
                return true;
            }
            else if ((_expr == null) && ((item is BoolArgument) || (item is UniversalArgument)))
            {
                _expr = item as ExprBase;
                _expr.ValueProvider = _provider;
                return true;
            }
            return false;
        }
        public override ICompilableObject Simplify()
        {
            _expr = _expr.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_expr != null)
                {
                    _expr.ValueProvider = value;
                }
            }
        }
        public override int AsInt
        {
            get
            {
                _provider.BeginIterator();
                int cnt = 0;
                do
                {
                    if (_expr.AsBool)
                    {
                        cnt++;
                    }
                } while (_provider.Next());
                _provider.EndIterator();
                return cnt;
            }
        }
        public override double AsDouble
        {
            get
            {
                return AsInt;
            }
        }
        public override Color AsColor
        {
            get
            {
                return Color.FromArgb(AsInt);
            }
        }
    }
    [Serializable]
    public class IntCountAggregate : CountAggregate
    {       
        public IntCountAggregate() : base()
        {
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.Int;
            }
        }
    }
    [Serializable]
    public class DoubleCountAggregate : CountAggregate
    {
        public DoubleCountAggregate() : base()
        {
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.Double;
            }
        }
    }
}
