﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class DoubleAggregateFunction : ArithmeticArgument
    {
        private ExprBase _expr = null;
        public DoubleAggregateFunction() : base()
        {
        }
        public override bool Indexed
        {
            get
            {
                return _expr.Indexed;
            }
        }
        public override string Text
        {
            get
            {
                return _text;
            }
            set
            {
            }
        }
        public override string AsText
        {
            get
            {
                if (_expr == null)
                {
                    return "DoubleAggregateFunction: " + _text;
                }
                return _text + "(" + _expr.AsText + ")";
            }
        }
        public override int OperandCount
        {
            get
            {
                return 1;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _expr;
                default:
                    return null;
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            if (item is FunctionName)
            {
                _text = item.Text;
                return true;
            }
            else if ((_expr == null) && ((item is ArithmeticArgument) || (item is UniversalArgument)))
            {
                _expr = item as ExprBase;
                _expr.ValueProvider = _provider;
                return true;
            }
            return false;
        }
        public override ICompilableObject Simplify()
        {
            _expr = _expr.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_expr != null)
                {
                    _expr.ValueProvider = value;
                }
            }
        }
        public override double AsDouble
        {
            get
            {
                _provider.BeginIterator();
                double fr = 0;
                switch (_text)
                {
                    case "dsum":
                        do
                        {
                            fr += _expr.AsDouble;
                        } while (_provider.Next());
                        break;
                    case "dprod":
                        fr = 1;
                        do
                        {
                            fr *= _expr.AsDouble;
                        } while (_provider.Next());
                        break;
                    case "dmax":
                        fr = int.MinValue;
                        do
                        {
                            fr = Math.Max(fr, _expr.AsDouble);
                        } while (_provider.Next());
                        break;
                    case "dmin":
                        fr = int.MaxValue;
                        do
                        {
                            fr = Math.Min(fr, _expr.AsDouble);
                        } while (_provider.Next());
                        break;
                    case "davg":
                        double cnt = 0;
                        do
                        {
                            fr += _expr.AsDouble;
                            cnt++;
                        } while (_provider.Next());
                        fr /= cnt;
                        break;
                    case "var":
                        cnt = 0;
                        double fr2 = 0;
                        do
                        {
                            double v = _expr.AsDouble;
                            fr += v;
                            fr2 += fr * fr;
                            cnt++;
                        } while (_provider.Next());
                        fr = (fr2 / cnt) - ((fr * fr) / cnt);
                        break;
                    case "svar":
                        cnt = 0;
                        fr2 = 0;
                        do
                        {
                            double v = _expr.AsDouble;
                            fr += v;
                            fr2 += fr * fr;
                            cnt++;
                        } while (_provider.Next());
                        fr = cnt * ((fr2 / cnt) - ((fr * fr) / cnt)) / (cnt - 1);
                        break;
                    case "sd":
                        cnt = 0;
                        fr2 = 0;
                        do
                        {
                            double v = _expr.AsDouble;
                            fr += v;
                            fr2 += fr * fr;
                            cnt++;
                        } while (_provider.Next());
                        fr = Math.Sqrt((fr2 / cnt) - ((fr * fr) / cnt));
                        break;
                    case "ssd":
                        cnt = 0;
                        fr2 = 0;
                        do
                        {
                            double v = _expr.AsDouble;
                            fr += v;
                            fr2 += fr * fr;
                            cnt++;
                        } while (_provider.Next());
                        fr = Math.Sqrt(cnt * ((fr2 / cnt) - ((fr * fr) / cnt)) / (cnt - 1));
                        break;
                    default:
                        throw new NotImplementedException();
                }
                _provider.EndIterator();
                return fr;
            }
        }
        public override int AsInt
        {
            get
            {
                return (int)AsDouble;
            }
        }
        public override Color AsColor
        {
            get
            {
                return Color.FromArgb(AsInt);
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.Double;
            }
        }
    }
}
