﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class DoubleFunction : ArithmeticArgument
    {
        private ExprBase _expr = null;

        public DoubleFunction() : base()
        {
        }
        public override bool Indexed
        {
            get
            {
                return _expr.Indexed;
            }
        }
        public override string Text
        {
            get
            {
                return _text;
            }
            set
            {
            }
        }
        public override string AsText
        {
            get
            {
                if (_expr == null)
                {
                    return "DoubleFunction: " + _text;
                }
                return _text + "(" + _expr.AsText + ")";
            }
        }
        public override int OperandCount
        {
            get
            {
                return 1;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _expr;
                default:
                    return null;
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            if (item is FunctionName)
            {
                _text = item.Text;
                return true;
            }
            else if ((_expr == null) && ((item is ArithmeticArgument) || (item is UniversalArgument)))
            {
                _expr = item as ExprBase;
                _expr.ValueProvider = _provider;
                return true;
            }
            return false;
        }
        public override ICompilableObject Simplify()
        {
            _expr = _expr.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_expr != null)
                {
                    _expr.ValueProvider = value;
                }
            }
        }
        public override int AsInt
        {
            get
            {
                return (int)AsDouble;
            }
        }
        public override Color AsColor
        {
            get
            {
                return Color.FromArgb(AsInt);
            }
        }
        public override double AsDouble
        {
            get
            {
                switch (_text)
                {
                    case "double":
                        return _expr.AsDouble;
                    case "dsign":
                        return Math.Sign(_expr.AsDouble);
                    case "dfloor":
                        return Math.Floor(_expr.AsDouble);
                    case "dceil":
                        return Math.Ceiling(_expr.AsDouble);
                    case "dabs":
                        return Math.Abs(_expr.AsDouble);
                    case "ln":
                        return Math.Log(_expr.AsDouble);
                    case "sqrt":
                        return Math.Sqrt(_expr.AsDouble);
                    case "exp":
                        return Math.Exp(_expr.AsDouble);
                    case "log":
                        return Math.Log10(_expr.AsDouble);
                    case "log2":
                        return Math.Log(_expr.AsDouble, 2);
                    case "sin":
                        return Math.Sin(_expr.AsDouble);
                    case "cos":
                        return Math.Cos(_expr.AsDouble);
                    case "tan":
                        return Math.Tan(_expr.AsDouble);
                    case "sinh":
                        return Math.Sinh(_expr.AsDouble);
                    case "cosh":
                        return Math.Cosh(_expr.AsDouble);
                    case "tanh":
                        return Math.Tanh(_expr.AsDouble);
                    case "hue":
                        return _expr.AsColor.GetHue();
                    case "sat":
                        return _expr.AsColor.GetSaturation();
                    case "bri":
                        return _expr.AsColor.GetBrightness();
                    default:
                        throw new NotImplementedException();
                }
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.Double;
            }
        }
    }
}
