﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;
using System.Windows.Forms;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public abstract class ExprBase : ICompilableObject, IExpressionValue
    {
        protected string _text;
        [NonSerialized]
        protected IValueProvider _provider = null;
        [NonSerialized]
        protected ITestProvider _testProvider = null;

        public ExprBase()
        {
        }
        public bool Parenthesis { get; set; }
        public virtual bool Indexed { get; }
        public virtual string AsText
        {
            get
            {
                return _text;
            }
        }
        public ITestProvider TestProvider
        {
            get
            {
                return _testProvider;
            }
            set
            {
                _testProvider = value;
            }
        }
        public virtual int OperandCount
        {
            get
            {
                return 0;
            }
        }
        public virtual ExprBase Operand(int n)
        {
            return null;
        }
        #region ICompilableObject
        public virtual string Text
        {
            get
            {
                return _text;
            }
            set
            {
                _text = value;
            }
        }
        public virtual bool AddItem(ICompilableObject item)
        {
            return false;
        }
        public virtual ICompilableObject Simplify()
        {
            return this;
        }
        public virtual void Test(Form parent)
        {
            if (_testProvider != null)
            {
                _testProvider.Test(this, parent);
            }
        }
        #endregion
        #region IExpressionValue
        public virtual IValueProvider ValueProvider
        {
            get
            {
                return _provider;
            }
            set
            {
                _provider = value;
            }
        }
        public abstract int AsInt { get; }
        public abstract double AsDouble { get; }
        public abstract string AsString { get; }
        public abstract bool AsBool { get; }
        public abstract Color AsColor { get; }
        public abstract ExprType PreferredType { get; }
        #endregion
        public override string ToString()
        {
            return AsText;
        }
    }
}
