﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;
using CommonObjects.Utils;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class IndexedProperty : UniversalArgument
    {
        private ExprBase _exprIx = null;
        private ExprType _typeDef = ExprType.None;

        public IndexedProperty() : base()
        {
        }
        public override string Text
        {
            get
            {
                return base.Text;
            }

            set
            {
            }
        }
        public override bool Indexed
        {
            get
            {
                return true;
            }
        }
        public override string AsText
        {
            get
            {
                if (_exprIx == null)
                {
                    return "[...].\\" + _text + ":" + TypedefText.AsText(_typeDef);
                }
                return "[" + _exprIx.AsText + "].\\" + _text + ":" + TypedefText.AsText(_typeDef);
            }
        }
        public override int OperandCount
        {
            get
            {
                return 1;
            }
        }
        public override ExprBase Operand(int n)
        {
            return _exprIx;
        }
        public override bool AddItem(ICompilableObject item)
        {
            if ((_exprIx == null) && ((item is ArithmeticArgument) || (item is UniversalArgument)))
            {
                _exprIx = item as ExprBase;
                return true;
            }
            if (item is PropertyReference)
            {
                _text = item.Text;
                _typeDef = (item as PropertyReference).PreferredType;
                return true;
            }
            return false;
        }
        public override ICompilableObject Simplify()
        {
            _exprIx = _exprIx.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (value != null)
                {
                    IVariantValue val = null;
                    if ((_exprIx != null) && !string.IsNullOrEmpty(_text))
                    {
                        _exprIx.ValueProvider = value;
                        val = value.GetIndexedPropertyValue(_exprIx.AsInt, _text);
                        if (val != null)
                        {
                            if (_typeDef == ExprType.None)
                            {
                                if ((val.NativeType == typeof(int)) || (val.NativeType == typeof(Color)))
                                {
                                    _typeDef = ExprType.Int;
                                }
                                else if (val.NativeType == typeof(double))
                                {
                                    _typeDef = ExprType.Double;
                                }
                                else if (val.NativeType == typeof(bool))
                                {
                                    _typeDef = ExprType.Bool;
                                }
                                else if (val.NativeType == typeof(string))
                                {
                                    _typeDef = ExprType.String;
                                }
                                else if (val.NativeType == typeof(Color))
                                {
                                    _typeDef = ExprType.Color;
                                }
                            }
                        }
                    }
                }
            }
        }
        public override int AsInt
        {
            get
            {
                return _provider.GetIndexedPropertyValue(_exprIx.AsInt, _text).AsInt;
            }
        }
        public override double AsDouble
        {
            get
            {
                return _provider.GetIndexedPropertyValue(_exprIx.AsInt, _text).AsDouble;
            }
        }
        public override Color AsColor
        {
            get
            {
                return _provider.GetIndexedPropertyValue(_exprIx.AsInt, _text).AsColor;
            }
        }
        public override bool AsBool
        {
            get
            {
                return _provider.GetIndexedPropertyValue(_exprIx.AsInt, _text).AsBool;
            }
        }
        public override string AsString
        {
            get
            {
                return _provider.GetIndexedPropertyValue(_exprIx.AsInt, _text).AsString;
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return _typeDef;
            }
        }
    }
}
