﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    public class IntAggregateFunction : ArithmeticArgument
    {
        private ExprBase _expr = null;
        public IntAggregateFunction() : base()
        {
        }
        public override bool Indexed
        {
            get
            {
                return _expr.Indexed;
            }
        }
        public override string Text
        {
            get
            {
                return _text;
            }
            set
            {
            }
        }
        public override string AsText
        {
            get
            {
                if (_expr == null)
                {
                    return "IntAggregateFunction: " + _text;
                }
                return _text + "(" + _expr.AsText + ")";
            }
        }
        public override int OperandCount
        {
            get
            {
                return 1;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _expr;
                default:
                    return null;
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            if (item is FunctionName)
            {
                _text = item.Text;
                return true;
            }
            else if ((_expr == null) && ((item is ArithmeticArgument) || (item is UniversalArgument)))
            {
                _expr = item as ExprBase;
                _expr.ValueProvider = _provider;
                return true;
            }
            return false;
        }
        public override ICompilableObject Simplify()
        {
            _expr = _expr.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_expr != null)
                {
                    _expr.ValueProvider = value;
                }
            }
        }
        public override int AsInt
        {
            get
            {
                _provider.BeginIterator();
                int fr = 0;
                switch (_text)
                {
                    case "isum":
                        do
                        {
                            fr += _expr.AsInt;
                        } while (_provider.Next());
                        break;
                    case "iprod":
                        fr = 1;
                        do
                        {
                            fr *= _expr.AsInt;
                        } while (_provider.Next());
                        break;
                    case "imax":
                        fr = int.MinValue;
                        do
                        {
                            fr = Math.Max(fr, _expr.AsInt);
                        } while (_provider.Next());
                        break;
                    case "imin":
                        fr = int.MaxValue;
                        do
                        {
                            fr = Math.Min(fr, _expr.AsInt);
                        } while (_provider.Next());
                        break;
                    case "iavg":
                        int cnt = 0;
                        do
                        {
                            fr += _expr.AsInt;
                            cnt++;
                        } while (_provider.Next());
                        fr /= cnt;
                        break;
                    default:
                        throw new NotImplementedException();
                }
                _provider.EndIterator();
                return fr;
            }
        }
        public override double AsDouble
        {
            get
            {
                return AsInt;
            }
        }
        public override Color AsColor
        {
            get
            {
                return Color.FromArgb(AsInt);
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.Int;
            }
        }
    }
}
