﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class IntFunction : ArithmeticArgument
    {
        private ExprBase _expr = null;

        public IntFunction() : base()
        {
        }
        public override bool Indexed
        {
            get
            {
                return _expr.Indexed;
            }
        }
        public override string Text
        {
            get
            {
                return _text;
            }
            set
            {
            }
        }
        public override string AsText
        {
            get
            {
                if (_expr == null)
                {
                    return "IntFunction: " + _text;
                }
                return _text + "(" + _expr.AsText + ")";
            }
        }
        public override int OperandCount
        {
            get
            {
                return 1;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _expr;
                default:
                    return null;
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            if (item is FunctionName)
            {
                _text = item.Text;
                return true;
            }
            else if ((_expr == null) && ((item is ArithmeticArgument) || (item is UniversalArgument)))
            {
                _expr = item as ExprBase;
                _expr.ValueProvider = _provider;
                return true;
            }
            return false;
        }
        public override ICompilableObject Simplify()
        {
            _expr = _expr.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_expr != null)
                {
                    _expr.ValueProvider = value;
                }
            }
        }
        public override int AsInt
        {
            get
            {
                switch (_text)
                {
                    case "int":
                        return _expr.AsInt;
                    case "isign":
                        return Math.Sign(_expr.AsInt);
                    case "ifloor":
                        return (int)Math.Floor(_expr.AsDouble);
                    case "iceil":
                        return (int)Math.Ceiling(_expr.AsDouble);
                    case "iabs":
                        return Math.Abs(_expr.AsInt);
                    case "red":
                        return _expr.AsColor.R;
                    case "green":
                        return _expr.AsColor.G;
                    case "blue":
                        return _expr.AsColor.B;
                    default:
                        throw new NotImplementedException();
                }
            }
        }
        public override double AsDouble
        {
            get
            {
                return AsInt;
            }
        }
        public override Color AsColor
        {
            get
            {
                return Color.FromArgb(AsInt);
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.Int;
            }
        }
    }
}
