﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class LogicExpression : BoolArgument
    {
        private ExprBase _exp1 = null;
        private ExprBase _exp2 = null;
        private BoolOperator _op = null;

        public LogicExpression()
        {
        }
        public override bool Indexed
        {
            get
            {
                if (_exp2 == null)
                {
                    return _exp1.Indexed;
                }
                return _exp1.Indexed || _exp2.Indexed;
            }
        }
        public override string AsText
        {
            get
            {
                if (_exp1 == null)
                {
                    return "LogicExpression";
                }
                string paro = Parenthesis ? "(" : "";
                string parc = Parenthesis ? ")" : "";
                if (_exp2 == null)
                {
                    if (_op == null)
                    {
                        return paro + _exp1.AsText + parc;
                    }
                    else
                    {
                        return paro + _op.Text + _exp1.AsText + parc;
                    }
                }
                else
                {
                    return paro + _exp1.AsText + _op.Text + _exp2.AsText + parc;
                }
            }
        }
        public override int OperandCount
        {
            get
            {
                return 2;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _exp1;
                case 1:
                    return _exp2;
                default:
                    return null;
            }
        }
        private void GroupLeft()
        {
            if ((_exp1 != null) && (_op != null) && (_exp2 != null) && (_exp2 is LogicExpression))
            {
                LogicExpression exp = _exp2 as LogicExpression;
                if ((!exp.Parenthesis) && (exp._op != null) && (exp._exp2 != null) && (_op.Precedence == exp._op.Precedence))
                {
                    LogicExpression exp1 = new LogicExpression();
                    exp1.ValueProvider = _provider;
                    exp1._exp1 = _exp1;
                    exp1._op = _op;
                    exp1._exp2 = exp._exp1;
                    _exp1 = exp1;
                    _op = exp._op;
                    _exp2 = exp._exp2;
                }
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            bool accept = false;
            if ((_exp1 == null) && ((item is BoolArgument) || (item is UniversalArgument)))
            {
                _exp1 = item as ExprBase;
                _exp1.ValueProvider = _provider;
                accept = true;
            }
            else if ((_op == null) && (item is BoolOperator))
            {
                _op = item as BoolOperator;
                accept = true;
            }
            else if ((_exp2 == null) && ((item is BoolArgument) || (item is UniversalArgument)))
            {
                _exp2 = item as ExprBase;
                _exp2.ValueProvider = _provider;
                accept = true;
            }
            if (accept)
            {
                GroupLeft();
            }
            return accept;
        }
        public override ICompilableObject Simplify()
        {
            if (_exp1 != null)
            {
                _exp1 = _exp1.Simplify() as ExprBase;
                if ((_exp2 == null) && (_op == null))
                {
                    LogicExpression ex1 = _exp1 as LogicExpression;
                    if (ex1 != null)
                    {
                        _exp2 = ex1._exp2;
                        _op = ex1._op;
                        _exp1 = ex1._exp1;
                        Parenthesis = Parenthesis || ex1.Parenthesis;
                    }
                    else
                    {
                        return _exp1;
                    }
                }
                else if (_exp2 != null)
                {
                    _exp2 = _exp2.Simplify() as ExprBase;
                }
                GroupLeft();
            }
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_exp1 != null)
                {
                    _exp1.ValueProvider = value;
                }
                if (_exp2 != null)
                {
                    _exp2.ValueProvider = value;
                }
            }
        }
        public override bool AsBool
        {
            get
            {
                if (_exp2 == null)
                {
                    if (_op != null)
                    {
                        return _op.Operate(_exp1.AsBool);
                    }
                    return _exp1.AsBool;
                }
                return _op.Operate(_exp1.AsBool, _exp2.AsBool);
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.Bool;
            }
        }
    }
}
