﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;
using System.Threading;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class Number : ArithmeticArgument
    {
        private double _dvalue;
        private int _ivalue;
        private ExprType _pType = ExprType.Int;
        public Number() : base()
        {
        }
        public override string Text
        {
            get
            {
                return base.Text;
            }
            set
            {
                if (!value.Contains('.'))
                {
                    if (int.TryParse(value, out _ivalue))
                    {
                        _dvalue = _ivalue;
                    }
                    else
                    {
                        _dvalue = double.Parse(value);
                        _pType = ExprType.Double;
                    }
                }
                else
                {
                    string sep = Thread.CurrentThread.CurrentCulture.NumberFormat.NumberDecimalSeparator;
                    _dvalue = double.Parse(value.Replace(".", sep));
                    _ivalue = (int)_dvalue;
                    _pType = ExprType.Double;
                }
                base.Text = value;
            }
        }
        public override int AsInt
        {
            get
            {
                return _ivalue;
            }
        }
        public override double AsDouble
        {
            get
            {
                return _dvalue;
            }
        }
        public override Color AsColor
        {
            get
            {
                return Color.FromArgb(AsInt);
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return _pType;
            }
        }
        public override string ToString()
        {
            return Text; 
        }
    }
}
