﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using BNFUP.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class Operator : ICompilableObject
    {
        protected string _operator = "";

        public Operator()
        {
        }
        public Operator(string op)
        {
            _operator = op;
        }
        #region ICompilableObject
        public string Text
        {
            get
            {
                return _operator;
            }
            set
            {
                _operator = value;
            }
        }
        public bool AddItem(ICompilableObject item)
        {
            return false;
        }
        public ICompilableObject Simplify()
        {
            switch (_operator)
            {
                case "+":
                case "-":
                case "*":
                case "/":
                case "^":
                case "%":
                case "?":
                case "@":
                case "#":
                    return new ArithmeticOperator(_operator);
                case "&":
                case "|":
                case "!":
                    return new BoolOperator(_operator);
                default:
                    return new RelOperator(_operator);
            }
        }
        public void Test(Form main)
        {
            throw new NotImplementedException();
        }
        #endregion
        public virtual int Precedence { get; }
        public virtual double Operate(double v)
        {
            throw new NotImplementedException();
        }
        public virtual int Operate(int v)
        {
            throw new NotImplementedException();
        }
        public virtual bool Operate(bool v)
        {
            throw new NotImplementedException();
        }
        public virtual double Operate(double v1, double v2)
        {
            throw new NotImplementedException();
        }
        public virtual int Operate(int v1, int v2)
        {
            throw new NotImplementedException();
        }
        public virtual bool Operate(bool v1, bool v2)
        {
            throw new NotImplementedException();
        }
        public override string ToString()
        {
            return _operator;
        }
    }
    [Serializable]
    public class ArithmeticOperator : Operator
    {
        public ArithmeticOperator() : base()
        {
        }
        public ArithmeticOperator(string op) : base(op)
        {
        }
        public override int Precedence
        {
            get
            {
                switch (_operator)
                {
                    case "-":
                    case "+":
                    case "@":
                    case "#":
                        return 3;
                    case "*":
                    case "/":
                    case "%":
                    case "?":
                        return 2;
                    case "^":
                        return 1;
                    default:
                        return -1;
                }
            }
        }
        public override double Operate(double v)
        {
            if (_operator == "-")
            {
                return -v;
            }
            return v;
        }
        public override int Operate(int v)
        {
            if (_operator == "-")
            {
                return -v;
            }
            return v;
        }
        public override double Operate(double v1, double v2)
        {
            switch (_operator)
            {
                case "-":
                    return v1 - v2;
                case "+":
                    return v1 + v2;
                case "*":
                    return v1 * v2;
                case "/":
                    return v1 / v2;
                case "%":
                    return v1 % v2;
                case "^":
                    return Math.Pow(v1, v2);
                case "?":
                    return (int)v1 & (int)v2;
                case "@":
                    return (int)v1 | (int)v2;
                case "#":
                    return (int)v1 ^ (int)v2;
                default:
                    return double.NaN;
            }
        }
        public override int Operate(int v1, int v2)
        {
            switch (_operator)
            {
                case "-":
                    return v1 - v2;
                case "+":
                    return v1 + v2;
                case "*":
                    return v1 * v2;
                case "/":
                    return v1 / v2;
                case "%":
                    return v1 % v2;
                case "^":
                    return (int)Math.Pow(v1, v2);
                case "?":
                    return v1 & v2;
                case "@":
                    return v1 | v2;
                case "#":
                    return v1 ^ v2;
                default:
                    throw new NotImplementedException();
            }
        }
    }
    [Serializable]
    public class BoolOperator : Operator
    {
        public BoolOperator() : base()
        {
        }
        public BoolOperator(string op) : base(op)
        {
        }
        public override int Precedence
        {
            get
            {
                switch (_operator)
                {
                    case "|":
                        return 2;
                    case "&":
                        return 1;
                    default:
                        return -1;
                }
            }
        }
        public override bool Operate(bool v)
        {
            if (_operator == "!")
            {
                return !v;
            }
            return v;
        }
        public override bool Operate(bool v1, bool v2)
        {
            switch (_operator)
            {
                case "|":
                    return v1 || v2;
                case "&":
                    return v1 && v2;
                default:
                    throw new NotImplementedException();
            }
        }
    }
    [Serializable]
    public class RelOperator : Operator
    {
        public RelOperator() : base()
        {
        }
        public RelOperator(string op) : base(op)
        {
        }
        public override int Precedence
        {
            get
            {
                return 1;
            }
        }
        public bool BoolOperate(double v1, double v2)
        {
            switch (_operator)
            {
                case "<<":
                    return v1 < v2;
                case "<=":
                    return v1 <= v2;
                case "==":
                    return v1 == v2;
                case ">=":
                    return v1 >= v2;
                case ">>":
                    return v1 > v2;
                case "<>":
                    return v1 != v2;
                default:
                    throw new NotImplementedException();
            }
        }
        public bool BoolOperate(int v1, int v2)
        {
            switch (_operator)
            {
                case "<<":
                    return v1 < v2;
                case "<=":
                    return v1 <= v2;
                case "==":
                    return v1 == v2;
                case ">=":
                    return v1 >= v2;
                case ">>":
                    return v1 > v2;
                case "<>":
                    return v1 != v2;
                default:
                    throw new NotImplementedException();
            }
        }
    }
}
