﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class RelExpression : BoolArgument
    {
        private ExprBase _exp1 = null;
        private ExprBase _exp2 = null;
        private RelOperator _operator = null;
        public RelExpression() : base()
        {
        }
        public override bool Indexed
        {
            get
            {
                return _exp1.Indexed || _exp2.Indexed;
            }
        }
        public override string AsText
        {
            get
            {
                string paro = Parenthesis ? "(" : "";
                string parc = Parenthesis ? ")" : "";
                if (_exp1 == null)
                {
                    return paro + "RelExpression" + parc;
                }
                if (_operator == null)
                {
                    return paro + _exp1.AsText + "..." + parc;
                }
                if (_exp2 == null)
                {
                    return paro + _exp1.AsText + _operator.Text + "..." + parc;
                }
                return paro + _exp1.AsText + _operator.Text + _exp2.AsText + parc;
            }
        }
        public override int OperandCount
        {
            get
            {
                return 2;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _exp1;
                case 1:
                    return _exp2;
                default:
                    return null;
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            bool accept = false;
            if ((_exp1 == null) && ((item is ArithmeticArgument) || (item is UniversalArgument)))
            {
                _exp1 = item as ExprBase;
                _exp1.ValueProvider = _provider;
                accept = true;
            }
            else if ((_operator == null) && (item is RelOperator))
            {
                _operator = item as RelOperator;
                accept = true;
            }
            else if ((_exp2 == null) && ((item is ArithmeticArgument) || (item is UniversalArgument)))
            {
                _exp2 = item as ExprBase;
                _exp2.ValueProvider = _provider;
                accept = true;
            }
            return accept;
        }
        public override ICompilableObject Simplify()
        {
            _exp1 = _exp1.Simplify() as ExprBase;
            _exp2 = _exp2.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_exp1 != null)
                {
                    _exp1.ValueProvider = value;
                }
                if (_exp2 != null)
                {
                    _exp2.ValueProvider = value;
                }
            }
        }
        public override bool AsBool
        {
            get
            {
                if ((_exp1.PreferredType == ExprType.Int) && (_exp2.PreferredType == ExprType.Int))
                {
                    return _operator.BoolOperate(_exp1.AsInt, _exp2.AsInt);
                }
                return _operator.BoolOperate(_exp1.AsDouble, _exp2.AsDouble);
            }
        }
    }
}
