﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;


namespace CommonObjects.Expressions
{
    [Serializable]
    public class StrBoolFunction : BoolArgument
    {
        private ExprBase _expr1 = null;
        private ExprBase _expr2 = null;

        public StrBoolFunction() : base()
        {
        }
        public override bool Indexed
        {
            get
            {
                return _expr1.Indexed || _expr2.Indexed;
            }
        }
        public override string Text
        {
            get
            {
                return _text;
            }
            set
            {
            }
        }
        public override string AsText
        {
            get
            {
                if (_expr1 == null)
                {
                    return "StrBoolExpression: " + _text;
                }
                if (_expr2 == null)
                {
                    return _text + "(" + _expr1.AsText + ",...)";
                }
                return _text + "(" + _expr1.AsText + ", " + _expr2.AsText + ")";
            }
        }
        public override int OperandCount
        {
            get
            {
                return 2;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _expr1;
                case 1:
                    return _expr2;
                default:
                    return null;
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            if (item is FunctionName)
            {
                _text = item.Text;
                return true;
            }
            else if ((_expr1 == null) && ((item is StringArgument) || (item is UniversalArgument)))
            {
                _expr1 = item as ExprBase;
                _expr1.ValueProvider = _provider;
                return true;
            }
            else if ((_expr2 == null) && ((item is StringArgument) || (item is UniversalArgument)))
            {
                _expr2 = item as ExprBase;
                _expr2.ValueProvider = _provider;
                return true;
            }
            return false;
        }
        public override ICompilableObject Simplify()
        {
            _expr1 = _expr1.Simplify() as ExprBase;
            _expr2 = _expr2.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_expr1 != null)
                {
                    _expr1.ValueProvider = value;
                }
                if (_expr2 != null)
                {
                    _expr2.ValueProvider = value;
                }
            }
        }
        public override bool AsBool
        {
            get
            {
                switch (_text)
                {
                    case "streq":
                        return string.Compare(_expr1.AsString, _expr2.AsString, true) == 0;
                    case "streqcs":
                        return string.Compare(_expr1.AsString, _expr2.AsString, false) == 0;
                    case "cont":
                        return _expr1.AsString.Contains(_expr2.AsString);
                    case "stw":
                        return _expr1.AsString.StartsWith(_expr2.AsString);
                    case "endw":
                        return _expr1.AsString.EndsWith(_expr2.AsString);
                    case "match":
                        Regex reg = new Regex(_expr2.AsString);
                        return reg.IsMatch(_expr1.AsString);
                    default:
                        throw new NotImplementedException();
                }
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.Bool;
            }
        }
    }
}
