﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class StrIntFunction : ArithmeticArgument
    {
        private ExprBase _expr1 = null;
        private ExprBase _expr2 = null;

        public StrIntFunction() : base()
        {
        }
        public override bool Indexed
        {
            get
            {
                return _expr1.Indexed || _expr2.Indexed;
            }
        }
        public override string Text
        {
            get
            {
                return _text;
            }
            set
            {
            }
        }
        public override int OperandCount
        {
            get
            {
                return 2;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _expr1;
                case 1:
                    return _expr2;
                default:
                    return null;
            }
        }
        public override string AsText
        {
            get
            {
                if (_expr1 == null)
                {
                    return "StrIntFunction: " + _text;
                }
                if (_expr2 == null)
                {
                    return _text + "(" + _expr1.AsText + ",...)";
                }
                return _text + "(" + _expr1.AsText + ", " + _expr2.AsText + ")";
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            if (item is FunctionName)
            {
                _text = item.Text;
                return true;
            }
            else if ((_expr1 == null) && ((item is StringArgument) || (item is UniversalArgument)))
            {
                _expr1 = item as ExprBase;
                _expr1.ValueProvider = _provider;
                return true;
            }
            else if ((_expr2 == null) && ((item is StringArgument) || (item is UniversalArgument)))
            {
                _expr2 = item as ExprBase;
                _expr2.ValueProvider = _provider;
                return true;
            }
            return false;
        }
        public override ICompilableObject Simplify()
        {
            _expr1 = _expr1.Simplify() as ExprBase;
            _expr2 = _expr2.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_expr1 != null)
                {
                    _expr1.ValueProvider = value;
                }
                if (_expr2 != null)
                {
                    _expr2.ValueProvider = value;
                }
            }
        }
        public override int AsInt
        {
            get
            {
                switch (_text)
                {
                    case "cmp":
                        return _expr1.AsString.CompareTo(_expr2.AsString);
                    case "pos":
                        return _expr1.AsString.IndexOf(_expr2.AsString);
                    default:
                        throw new NotImplementedException();
                }
            }
        }
        public override double AsDouble
        {
            get
            {
                return AsInt;
            }
        }
        public override Color AsColor
        {
            get
            {
                throw new NotImplementedException();
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.Int;
            }
        }
    }
}
