﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class StrSubstringFunction : StringArgument
    {
        private ExprBase _sexpr = null;
        private ExprBase _expr1 = null;
        private ExprBase _expr2 = null;

        public StrSubstringFunction() : base()
        {
        }
        public override bool Indexed
        {
            get
            {
                return _sexpr.Indexed || _expr1.Indexed || _expr2.Indexed;
            }
        }
        public override string Text
        {
            get
            {
                return _text;
            }
            set
            {
            }
        }
        public override string AsText
        {
            get
            {
                if (_sexpr == null)
                {
                    return "StrSubstringFunction";
                }
                if (_expr1 == null)
                {
                    return _text + "(" + _sexpr.AsText + ",...,...)";
                }
                if (_expr2 == null)
                {
                    return _text + "(" + _sexpr.AsText + ", " + _expr1.AsText + ",...)";
                }
                return _text + "(" + _sexpr.AsText + ", " + _expr1.AsText + ", " + _expr2.AsText + ")";
            }
        }
        public override int OperandCount
        {
            get
            {
                return 3;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _sexpr;
                case 1:
                    return _expr1;
                case 2:
                    return _expr2;
                default:
                    return null;
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            if (item is FunctionName)
            {
                _text = item.Text;
                return true;
            }
            else if ((_sexpr == null) && ((item is StringArgument) || (item is UniversalArgument)))
            {
                _sexpr = item as ExprBase;
                _sexpr.ValueProvider = _provider;
                return true;
            }
            else if ((_expr1 == null) && (item is ArithmeticArgument))
            {
                _expr1 = item as ExprBase;
                _expr1.ValueProvider = _provider;
                return true;
            }
            else if ((_expr2 == null) && (item is ArithmeticArgument))
            {
                _expr2 = item as ExprBase;
                _expr2.ValueProvider = _provider;
                return true;
            }
            return false;
        }
        public override ICompilableObject Simplify()
        {
            _sexpr = _sexpr.Simplify() as ExprBase;
            _expr1 = _expr1.Simplify() as ExprBase;
            _expr2 = _expr2.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_sexpr != null)
                {
                    _sexpr.ValueProvider = value;
                }
                if (_expr1 != null)
                {
                    _expr1.ValueProvider = value;
                }
                if (_expr2 != null)
                {
                    _expr2.ValueProvider = value;
                }
            }
        }
        public override string AsString
        {
            get
            {
                try
                {
                    return _sexpr.AsString.Substring(_expr1.AsInt, _expr2.AsInt);
                }
                catch
                {
                    return "";
                }
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.String;
            }
        }
    }
    [Serializable]
    public class StrConcatFunction : StringArgument
    {
        private ExprBase _expr1 = null;
        private ExprBase _expr2 = null;

        public StrConcatFunction() : base()
        {
        }
        public override string Text
        {
            get
            {
                return _text;
            }
            set
            {
            }
        }
        public override string AsText
        {
            get
            {
                if (_expr1 == null)
                {
                    return "StrConcatFunction";
                }
                if (_expr2 == null)
                {
                    return _text + "(" + _expr1.AsText + ",...)";
                }
                return _text + "(" + _expr1.AsText + ", " + _expr2.AsText + ")";
            }
        }
        public override int OperandCount
        {
            get
            {
                return 2;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _expr1;
                case 1:
                    return _expr2;
                default:
                    return null;
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            if (item is FunctionName)
            {
                _text = item.Text;
                return true;
            }
            else if ((_expr1 == null) && ((item is StringArgument) || (item is UniversalArgument)))
            {
                _expr1 = item as ExprBase;
                _expr1.ValueProvider = _provider;
                return true;
            }
            else if ((_expr2 == null) && ((item is StringArgument) || (item is UniversalArgument)))
            {
                _expr2 = item as ExprBase;
                _expr2.ValueProvider = _provider;
                return true;
            }
            return false;
        }
        public override ICompilableObject Simplify()
        {
            _expr1 = _expr1.Simplify() as ExprBase;
            _expr2 = _expr2.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_expr1 != null)
                {
                    _expr1.ValueProvider = value;
                }
                if (_expr2 != null)
                {
                    _expr2.ValueProvider = value;
                }
            }
        }
        public override string AsString
        {
            get
            {
                return _expr1.AsString + _expr2.AsString;
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.String;
            }
        }
    }
    [Serializable]
    public class StrReplaceFunction : StringArgument
    {
        private ExprBase _expr1 = null;
        private ExprBase _expr2 = null;
        private ExprBase _expr3 = null;

        public StrReplaceFunction() : base()
        {
        }
        public override string Text
        {
            get
            {
                return _text;
            }
            set
            {
            }
        }
        public override string AsText
        {
            get
            {
                if (_expr1 == null)
                {
                    return "StrReplaceFunction";
                }
                if (_expr2 == null)
                {
                    return _text + "(" + _expr1.AsText + ",...,...)";
                }
                if (_expr3 == null)
                {
                    return _text + "(" + _expr1.AsText + ", " + _expr2.AsText + ",...)";
                }
                return _text + "(" + _expr1.AsText + ", " + _expr2.AsText + ", " + _expr3.AsText + ")";
            }
        }
        public override int OperandCount
        {
            get
            {
                return 3;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _expr1;
                case 1:
                    return _expr2;
                case 2:
                    return _expr3;
                default:
                    return null;
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            if (item is FunctionName)
            {
                _text = item.Text;
                return true;
            }
            else if ((_expr1 == null) && ((item is StringArgument) || (item is UniversalArgument)))
            {
                _expr1 = item as ExprBase;
                _expr1.ValueProvider = _provider;
                return true;
            }
            else if ((_expr2 == null) && ((item is StringArgument) || (item is UniversalArgument)))
            {
                _expr2 = item as ExprBase;
                _expr2.ValueProvider = _provider;
                return true;
            }
            else if ((_expr3 == null) && ((item is StringArgument) || (item is UniversalArgument)))
            {
                _expr3 = item as ExprBase;
                _expr3.ValueProvider = _provider;
                return true;
            }
            return false;
        }
        public override ICompilableObject Simplify()
        {
            _expr1 = _expr1.Simplify() as ExprBase;
            _expr2 = _expr2.Simplify() as ExprBase;
            _expr3 = _expr3.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_expr1 != null)
                {
                    _expr1.ValueProvider = value;
                }
                if (_expr2 != null)
                {
                    _expr2.ValueProvider = value;
                }
                if (_expr3 != null)
                {
                    _expr3.ValueProvider = value;
                }
            }
        }
        public override string AsString
        {
            get
            {
                return _expr1.AsString.Replace(_expr2.AsString, _expr3.AsString);
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.String;
            }
        }
    }
}
