﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class TruthFunction : BoolArgument
    {
        private ExprBase _expr = null;

        public TruthFunction() : base()
        {
        }
        public override bool Indexed
        {
            get
            {
                return _expr.Indexed;
            }
        }
        public override string Text
        {
            get
            {
                return _text;
            }
            set
            {
            }
        }
        public override string AsText
        {
            get
            {
                if (_expr == null)
                {
                    return "TruthFunction: " + _text;
                }
                return _text + "(" + _expr.AsText + ")";
            }
        }
        public override int OperandCount
        {
            get
            {
                return 1;
            }
        }
        public override ExprBase Operand(int n)
        {
            switch (n)
            {
                case 0:
                    return _expr;
                default:
                    return null;
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            if (item is FunctionName)
            {
                _text = item.Text;
                return true;
            }
            else if ((_expr == null) && (item is UniversalArgument))
            {
                _expr = item as ExprBase;
                _expr.ValueProvider = _provider;
                return true;
            }
            return false;
        }
        public override ICompilableObject Simplify()
        {
            _expr = _expr.Simplify() as ExprBase;
            return this;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if (_expr != null)
                {
                    _expr.ValueProvider = value;
                }
            }
        }
        public override bool AsBool
        {
            get
            {
                switch (_text)
                {
                    case "ist":
                        return _expr.AsBool;
                    case "isf":
                        return !_expr.AsBool;
                    default:
                        throw new NotImplementedException();
                }
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return ExprType.Bool;
            }
        }
    }
}
