﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using BNFUP.Interfaces;
using CommonObjects.Interfaces;
using CommonObjects.Utils;

namespace CommonObjects.Expressions
{
    [Serializable]
    public class VariableReference : UniversalArgument
    {
        private ExprType _typeDef = ExprType.None;

        public VariableReference() : base()
        {
        }
        public override string Text
        {
            get
            {
                return base.Text;
            }
            set
            {
            }
        }
        public override string AsText
        {
            get
            {
                return "_" + _text + ":" + TypedefText.AsText(_typeDef);
            }
        }
        public override bool AddItem(ICompilableObject item)
        {
            if (item is TypeDef)
            {
                _typeDef = (item as TypeDef).TypeDefined;
                return true;
            }
            else if (item is VarIdentifier)
            {
                _text = item.Text;
                return true;
            }
            return false;
        }
        public override IValueProvider ValueProvider
        {
            get
            {
                return base.ValueProvider;
            }
            set
            {
                base.ValueProvider = value;
                if ((value != null) && !string.IsNullOrEmpty(_text))
                {
                    IVariantValue val = value.GetVarValue(_text);
                    if (val != null)
                    {
                        if (_typeDef == ExprType.None)
                        {
                            if ((val.NativeType == typeof(int)) || (val.NativeType == typeof(Color)))
                            {
                                _typeDef = ExprType.Int;
                            }
                            else if (val.NativeType == typeof(double))
                            {
                                _typeDef = ExprType.Double;
                            }
                            else if (val.NativeType == typeof(bool))
                            {
                                _typeDef = ExprType.Bool;
                            }
                            else if (val.NativeType == typeof(string))
                            {
                                _typeDef = ExprType.String;
                            }
                        }
                    }
                }
            }
        }
        public override int AsInt
        {
            get
            {
                return _provider.GetVarValue(_text).AsInt;
            }
        }
        public override double AsDouble
        {
            get
            {
                return _provider.GetVarValue(_text).AsDouble;
            }
        }
        public override bool AsBool
        {
            get
            {
                return _provider.GetVarValue(_text).AsBool;
            }
        }
        public override string AsString
        {
            get
            {
                return _provider.GetVarValue(_text).AsString;
            }
        }
        public override Color AsColor
        {
            get
            {
                return _provider.GetVarValue(_text).AsColor;
            }
        }
        public override ExprType PreferredType
        {
            get
            {
                return _typeDef;
            }
        }
    }
}
