﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CommonObjects.Interfaces
{
    /// <summary>
    /// Defines a class as a variant property
    /// </summary>
    public interface IObjectProperty
    {
        /// <summary>
        /// Gets or sets the property name
        /// </summary>
        string Name { get; set; }
        /// <summary>
        /// Gets the property value
        /// </summary>
        IVariantValue Value { get; }
        /// <summary>
        /// Gets or sets the default type of the property value, the type must implement the IVariantValue interface
        /// </summary>
        Type DefaultValueType { get; set; }
        /// <summary>
        /// Creates a variant value with the default type defined in the DefaultValueType property
        /// </summary>
        /// <param name="value">Value to assign to the result</param>
        /// <returns>An IVariantValue object with the provided value</returns>
        IVariantValue CreateValue(object value);
    }
}
