﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CommonObjects.Interfaces
{
    /// <summary>
    /// Property value provider during expression evaluation
    /// </summary>
    public interface IValueProvider
    {
        /// <summary>
        /// Gets the value of a variable
        /// </summary>
        /// <param name="name">Variable name</param>
        /// <returns>Variable value as an IVariantValue</returns>
        IVariantValue GetVarValue(string name);
        /// <summary>
        /// Gets the value of a property
        /// </summary>
        /// <param name="name">Property name</param>
        /// <returns>Property value as an IVariantValue</returns>
        IVariantValue GetPropertyValue(string name);
        /// <summary>
        /// Gets the value of a property into an iterator loop
        /// </summary>
        /// <param name="name">Name of the property</param>
        /// <returns>Property value for the current iterator position, as an IVariantValue</returns>
        IVariantValue GetIteratorPropertyValue(string name);
        /// <summary>
        /// Gets the value of an indexed property  
        /// </summary>
        /// <param name="ix">Index of the property</param>
        /// <param name="name">Property name</param>
        /// <returns>Property value as an IVariantValue</returns>
        IVariantValue GetIndexedPropertyValue(int ix, string name);
        /// <summary>
        /// Starts an iterator loop
        /// </summary>
        void BeginIterator();
        /// <summary>
        /// Advances the iterator
        /// </summary>
        /// <returns>false when the iterator is finished</returns>
        bool Next();
        /// <summary>
        /// Ends an interator loop operation
        /// </summary>
        void EndIterator();
    }
}
