﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;

namespace CommonObjects.Interfaces
{
    /// <summary>
    /// Variant value
    /// </summary>
    public interface IVariantValue
    {
        /// <summary>
        /// Indicates if the value can be changed
        /// </summary>
        bool Mutable { get; }
        /// <summary>
        /// Gets or sets the value as an integer
        /// </summary>
        int AsInt { get; set; }
        /// <summary>
        /// Gets or sets the value as a double
        /// </summary>
        double AsDouble { get; set; }
        /// <summary>
        /// Gets or sets the value as a color
        /// </summary>
        Color AsColor { get; set; }
        /// <summary>
        /// Gets or sets the value as a Color
        /// </summary>
        string AsString { get; set; }
        /// <summary>
        /// Gets or sets the value as a boolean
        /// </summary>
        bool AsBool { get; set; }
        /// <summary>
        /// Gets or sets the value as an object
        /// </summary>
        object AsObject { get; set; }
        /// <summary>
        /// Gets the real type of the data
        /// </summary>
        Type NativeType { get; }
        /// <summary>
        /// Creates a new IVariantValue from an object
        /// </summary>
        /// <param name="value">Value to assign to the result</param>
        /// <returns>IVariantValue object with the given value</returns>
        IVariantValue FromObject(object value);
        /// <summary>
        /// Creates a new IVariantValue with the same data type as the sample
        /// </summary>
        /// <param name="value">Sample value to get the data type</param>
        /// <returns>IVariantValue object with the given value and type</returns>
        IVariantValue ConvertFrom(IVariantValue value);
        /// <summary>
        /// Creates a copy of this object
        /// </summary>
        /// <returns>IVariantValue object with the same value as this</returns>
        IVariantValue Clone();
    }
}
