﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;
using System.Text;
using System.Threading.Tasks;
using CommonObjects.Interfaces;

namespace CommonObjects.Utils
{
    public class RandomNormal : IRandomBase
    {
        private double[] _stack = null;
        private double[] _sampler = null;
        private double _mean;
        private double _sd;
        private Random r = null;

        public RandomNormal(int resolution, int seed, double sdmax, double mean, double sd)
        {
            if (seed > 0)
            {
                r = new Random(seed);
            }
            else
            {
                r = new Random();
            }
            _mean = mean;
            _sd = sd;
            if (sdmax <= 0)
            {
                sdmax = 1;
            }
            _stack = new double[resolution];            
            ConstructNormal(_stack, resolution, sdmax, sd);
            _sampler = new double[1000];
            ConstructNormal(_sampler, 1000, 3, (double)resolution / 3);
        }
        public double Mean
        {
            get
            {
                return _mean;
            }
        }
        public double StandardDeviation
        {
            get
            {
                return _sd;
            }
        }
        private void ConstructNormal(double[] data, int resolution, double sdmax, double sd)
        {
            double xmax = Math.Round(sd * sdmax, 10);
            double gamma = Math.Round(sd * Math.Sqrt(2 * Math.PI), 10);
            double ymax = Math.Round(1 / gamma, 10);
            double ymin = Math.Round(ymax * Math.Exp(-0.5 * sdmax * sdmax), 10);
            double yincr = Math.Round((ymax - ymin) / (double)resolution, 10);
            data[0] = 0;
            double y = ymax - yincr;
            for (int layer = 1; layer < resolution; layer++)
            {                
                double x = Math.Round(sd * Math.Sqrt(-2 * Math.Log(gamma * y)), 10);
                data[layer] = x;                    
                y -= yincr;
            }
        }
        public double NextValue()
        {
            int slayer = r.Next(_sampler.Length);            
            int layer = (int)_sampler[slayer];
            double mult = r.NextDouble();
            double sgn = r.NextDouble() >= 0.5 ? 1 : -1;
            return Math.Round(_mean + (mult * _stack[layer] * sgn), 10);
        }
    }
}
