﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using CellularAutomata.Interfaces;
using WinCA.Properties;

namespace WinCA.Controls
{
    public partial class TransitionControl : UserControl
    {
        private bool _selected = false;
        private string _from = "";
        private string _to = "";
        private ICAStateProvider _provider = null;

        public TransitionControl()
        {
            InitializeComponent();
            opDelete.Text = UIResources.OPT_DELETETRAN;
            opEditar.Text = UIResources.OPT_EDITTRAN;
        }
        public event EventHandler OnSelectedChanged = null;
        public event EventHandler OnDeleteExpression = null;
        public event EventHandler OnEditExpression = null;
        public ICAStateProvider Provider
        {
            get
            {
                return _provider;
            }
            set
            {
                _provider = value;
                SetImage();
            }
        }
        public string From
        {
            get
            {
                return _from;
            }
            set
            {
                _from = value;
                SetImage();
            }
        }
        public string To
        {
            get
            {
                return _to;
            }
            set
            {
                _to = value;
                SetImage();
            }
        }
        public bool Selected
        {
            get
            {
                return _selected;
            }
            set
            {
                _selected = value;
                OnSelectedChanged?.Invoke(this, EventArgs.Empty);
            }
        }
        public override void Refresh()
        {
            base.Refresh();
            SetImage();
        }
        private void SetImage()
        {
            if (!string.IsNullOrEmpty(_from) && !string.IsNullOrEmpty(_to) && (_provider != null))
            {
                ICAState sts = _provider.GetState(_from);
                if (sts != null)
                {
                    if (!string.IsNullOrEmpty(sts.GetTransition(_to)))
                    {
                        pbTransition.Image = Resources.BMP_TRANSITION;
                        return;
                    }
                }
            }
            pbTransition.Image = null;
        }

        private void opDelete_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(_from) && !string.IsNullOrEmpty(_to) && (_provider != null))
            {
                OnDeleteExpression?.Invoke(this, EventArgs.Empty);
                SetImage();
            }
        }

        private void opEditar_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(_from) && !string.IsNullOrEmpty(_to) && (_provider != null))
            {
                OnEditExpression?.Invoke(this, EventArgs.Empty);
                SetImage();
            }
        }
    }
}
