﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using CellularAutomata.Interfaces;
using WinCA.Properties;

namespace WinCA.Controls
{
    public class TransitionMatrixPanel : Panel
    {
        private const float _fontSize = 10f;
        private const string _fontFamily = "Arial";
        private ICAStateProvider _provider = null;

        public TransitionMatrixPanel()
        {
        }
        public event TransitionEventHandler OnEditTransition = null;
        public event TransitionEventHandler OnDeleteTransition = null;
        public ICAStateProvider Provider
        {
            get
            {
                return _provider;
            }
            set
            {
                _provider = value;
                RefreshControls();
            }
        }
        public void RefreshControls()
        {
            Controls.Clear();
            if (_provider != null)
            {
                SuspendLayout();
                int w = (19 * _provider.Count);
                int x0 = 2 + ((Width - w) / 2);
                int y0 = 2 + (Height - (w + 20));
                for (int iy = 0; iy < _provider.Count; iy++)
                {
                    for (int ix = 0; ix < _provider.Count; ix++)
                    {
                        if (ix != iy)
                        {
                            TransitionControl ctl = new TransitionControl();
                            ctl.Provider = _provider;
                            ctl.OnDeleteExpression += new EventHandler(OnDeleteExpression);
                            ctl.OnEditExpression += new EventHandler(OnEditExpression);
                            ctl.OnSelectedChanged += new EventHandler(OnExpressionSelected);
                            ctl.From = _provider.GetState(iy).Name;
                            ctl.To = _provider.GetState(ix).Name;
                            Controls.Add(ctl);
                            ctl.Left = x0 + (19 * ix);
                            ctl.Top = y0 + (19 * iy);
                        }
                    }
                }
                ResumeLayout();
            }
            Refresh();
        }
        public override void Refresh()
        {
            base.Refresh();
            foreach (Control c in Controls)
            {
                c.Refresh();
            }
        }
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            if (_provider != null)
            {
                int w = (19 * _provider.Count);
                int x0 = ((Width - w) / 2);
                int y0 = Height - (w + 20);
                e.Graphics.DrawRectangle(Pens.DarkGray, new Rectangle(x0, y0, w, w));
                for (int iy = 1; iy < _provider.Count; iy++)
                {
                    for (int ix = 1; ix < _provider.Count; ix++)
                    {
                        e.Graphics.DrawLine(Pens.LightGray, new Point(x0 + (19 * ix), 1 + y0), new Point(x0 + (19 * ix), y0 + w - 1));
                        e.Graphics.DrawLine(Pens.LightGray, new Point(1 + x0, y0 + (19 * iy)), new Point(x0 + w - 1, y0 + (19 * iy)));
                    }
                }
                using (Font f = new Font(_fontFamily, _fontSize, FontStyle.Bold, GraphicsUnit.Pixel))
                {
                    float maxw = 0f;
                    for (int id = 0; id < _provider.Count; id++)
                    {
                        e.Graphics.FillRectangle(Brushes.DarkGray, new Rectangle(1 + x0 + (19 * id), 1 + y0 + (19 * id), 18, 18));
                        SizeF sz = e.Graphics.MeasureString(_provider.GetState(id).Name, f);
                        if (sz.Width > maxw)
                        {
                            maxw = sz.Width;
                        }
                    }
                    e.Graphics.DrawRectangle(Pens.Black, new Rectangle(x0 - ((int)maxw + 32), y0, (int)maxw + 32, w));
                    for (int tab = 0; tab < _provider.Count; tab++)
                    {
                        e.Graphics.FillRectangle(Brushes.LightGray, new Rectangle(x0 - ((int)maxw + 31), 1 + y0 + (19 * tab), (int)maxw + 32, 18));
                        using (Brush br = new SolidBrush(_provider.GetState(tab).StateColor))
                        {
                            e.Graphics.FillRectangle(br, new Rectangle(x0 - ((int)maxw + 30), 1 + y0 + (19 * tab), 16, 16));
                            e.Graphics.DrawRectangle(Pens.Black, new Rectangle(x0 - ((int)maxw + 30), 1 + y0 + (19 * tab), 16, 16));
                        }
                        SizeF sz = e.Graphics.MeasureString(_provider.GetState(tab).Name, f);
                        e.Graphics.DrawString(_provider.GetState(tab).Name, f, Brushes.Black, (float)x0 - sz.Width - 4f, (float)(1 + y0 + (19 * tab)) + (18f - sz.Height) / 2f);
                        if (tab > 0)
                        {
                            e.Graphics.DrawLine(Pens.Black, new Point(x0 - ((int)maxw + 32), y0 + (19 * tab)), new Point(x0 - 1, y0 + (19 * tab)));
                        }
                    }
                    e.Graphics.DrawRectangle(Pens.Black, new Rectangle(x0, y0 - ((int)maxw + 32), w, (int)maxw + 32));
                    for (int tab = 0; tab < _provider.Count; tab++)
                    {
                        e.Graphics.FillRectangle(Brushes.LightGray, new Rectangle(1 + x0 + (19 * tab), y0 - ((int)maxw + 31), 18, (int)maxw + 32));
                        using (Brush br = new SolidBrush(_provider.GetState(tab).StateColor))
                        {
                            e.Graphics.FillRectangle(br, new Rectangle(x0 + (19 * tab) + 1, y0 - 18, 16, 16));
                            e.Graphics.DrawRectangle(Pens.Black, new Rectangle(x0 + (19 * tab) + 1, y0 - 18, 16, 16));
                        }
                        Matrix m = e.Graphics.Transform;
                        e.Graphics.TranslateTransform(x0, y0);
                        e.Graphics.RotateTransform(-90f);
                        SizeF sz = e.Graphics.MeasureString(_provider.GetState(tab).Name, f);
                        e.Graphics.DrawString(_provider.GetState(tab).Name, f, Brushes.Black, 24f, (float)(1 + (19 * tab)) + (18f - sz.Height) / 2f);
                        e.Graphics.Transform = m;
                        if (tab > 0)
                        {
                            e.Graphics.DrawLine(Pens.Black, new Point(x0 + (19 * tab), y0), new Point(x0 + (19 * tab), y0 - ((int)maxw + 31)));
                        }
                    }
                }
                foreach (TransitionControl ctl in Controls)
                {
                    if (ctl.Selected)
                    {
                        e.Graphics.DrawRectangle(SystemPens.Highlight, new Rectangle(ctl.Left - 1, ctl.Top - 1, 17, 17));
                    }
                }
            }
        }
        protected override void OnResize(EventArgs eventargs)
        {
            base.OnResize(eventargs);
            RefreshControls();
        }
        private void UnselectAll(TransitionControl except)
        {
            foreach (TransitionControl ctl in Controls)
            {
                if ((ctl.Selected) && (ctl != except))
                {
                    ctl.Selected = false;
                }
            }
        }
        private void OnExpressionSelected(object sender, EventArgs e)
        {
            TransitionControl ctl = sender as TransitionControl;
            if (ctl != null)
            {
                UnselectAll(ctl);
                Refresh();
            }
        }
        private void OnDeleteExpression(object sender, EventArgs e)
        {
            TransitionControl ctl = sender as TransitionControl;
            if (ctl != null)
            {
                if (MessageBox.Show(string.Format(UIResources.MSG_DELETETRIGGER, ctl.From, ctl.To), UIResources.CAP_WARNING, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    ICAState sts = _provider.GetState(ctl.From);
                    if (sts != null)
                    {
                        if (!string.IsNullOrEmpty(sts.GetTransition(ctl.To)))
                        {
                            sts.SetTransition(ctl.To, null);
                            OnDeleteTransition?.Invoke(this, new TransitionEventArgs(ctl.From, ctl.To));
                        }
                    }
                }
            }
        }
        private void OnEditExpression(object sender, EventArgs e)
        {
            TransitionControl ctl = sender as TransitionControl;
            if (ctl != null)
            {
                OnEditTransition?.Invoke(this, new TransitionEventArgs(ctl.From, ctl.To));
            }
        }
    }
    public class TransitionEventArgs : EventArgs
    {
        public TransitionEventArgs(string from, string to)
        {
            From = from;
            To = to;
        }
        public string From { get;  private set; }
        public string To { get; private set; }
    }
    public delegate void TransitionEventHandler(object sender, TransitionEventArgs e);
}
