﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using CommonObjects.Interfaces;
using WinCA.Interfaces;
using WinCA.Properties;
using WinCA.Utils;

namespace WinCA.Controls
{
    public partial class VariantValueListEditor : UserControl
    {
        private IPropertyProvider _provider = null;
        public VariantValueListEditor()
        {
            InitializeComponent();
            foreach (DataEditor ed in DataEditor.EditorTypes)
            {
                cbDataType.Items.Add(ed);
            }
            cbDataType.SelectedIndex = 0;
        }
        public event PropertyChangeEventHandler PropertyChanged = null;
        public string ValueName
        {
            get
            {
                return lbData.Text;
            }
            set
            {
                lName.Text = value;
                bAdd.ToolTipText = string.Format(UIResources.TTP_NEWVALUE, value);
                bDelete.ToolTipText = string.Format(UIResources.TTP_DELETEVALUE, value);
            }
        }
        public IPropertyProvider Provider
        {
            get
            {
                return _provider;
            }
            set
            {
                _provider = value;
                GetValues();
            }
        }
        private string NewName
        {
            get
            {
                string name = lName.Text;
                int n = 1;
                while (TestName(name + n.ToString()) > 0)
                {
                    n++;
                }
                return name + n.ToString();
            }
        }
        private int TestName(string name)
        {
            int nc = 0;
            foreach (IObjectProperty p in lbData.Items)
            {
                if (string.Compare(p.Name, name, true) == 0)
                {
                    nc++;
                }
            }
            return nc;
        }        
        private void GetValues()
        {
            lbData.Items.Clear();
            if (_provider != null)
            {
                foreach (IObjectProperty p in _provider.Properties)
                {
                    lbData.Items.Add(p);
                }
            }
        }
        private void ChangeValue(string name, IObjectProperty p, bool select)
        {
            for (int ix = 0; ix < lbData.Items.Count; ix++)
            {
                if (string.Compare((lbData.Items[ix] as IObjectProperty).Name, name, true) == 0)
                {
                    lbData.Items[ix] = p;
                    if (select)
                    {
                        lbData.SelectedIndex = ix;
                    }
                    return;
                }
            }
        }

        private void lbData_SelectedIndexChanged(object sender, EventArgs e)
        {
            IObjectProperty value = lbData.SelectedItem as IObjectProperty;
            if (value != null)
            {
                DataEditor editor = DataEditor.FromVariantValue(value);
                pgValue.SelectedObject = editor;
            }
            else
            {
                pgValue.SelectedObject = null;
            }
        }

        private void pgValue_PropertyValueChanged(object s, PropertyValueChangedEventArgs e)
        {
            DataEditor pe = pgValue.SelectedObject as DataEditor;
            if (pe != null)
            {
                switch (e.ChangedItem.PropertyDescriptor.Name)
                {
                    case "Name":
                        if (string.IsNullOrEmpty(pe.Name) || TestName(pe.Name) > 0)
                        {
                            MessageBox.Show(UIResources.ERR_DUPLICATENAME);
                            pe.Name = e.OldValue.ToString();
                            pgValue.Refresh();
                        }
                        else
                        {
                            pe.Property.Name = pe.Name;
                            lbData.Items[lbData.SelectedIndex] = pe.Property;
                            PropertyChanged?.Invoke(this, new PropertyChangeEventArgs(pe.Property, _provider, PropertyChangeEventArgs.Change.NameChanged, e.OldValue.ToString()));
                        }                        
                        break;
                    case "DataType":                    
                        DataEditor dt = DataEditor.FromDataType(pe.DataType);
                        IObjectProperty pn = _provider.CreateProperty(pe.Name, dt.DefaultValue);
                        dt.Property = pn;
                        _provider.ChangeProperty(pe.Name, pn);
                        ChangeValue(pe.Name, pn, true);
                        pgValue.SelectedObject = dt;
                        PropertyChanged?.Invoke(this, new PropertyChangeEventArgs(pn, _provider, PropertyChangeEventArgs.Change.TypeChanged, pe.Name));
                        break;
                    case "Value":
                        dt = DataEditor.FromDataType(pe.DataType);
                        pn = _provider.CreateProperty(pe.Name, pe.UValue);
                        dt.Property = pn;
                        _provider.ChangeProperty(pe.Name, pn);
                        ChangeValue(pe.Name, pn, true);
                        pgValue.SelectedObject = dt;
                        PropertyChanged?.Invoke(this, new PropertyChangeEventArgs(pn, _provider, PropertyChangeEventArgs.Change.TypeChanged, pe.Name));
                        break;
                }
            }
        }

        private void bAdd_Click(object sender, EventArgs e)
        {
            DataEditor de = cbDataType.SelectedItem as DataEditor;
            IObjectProperty p = _provider.CreateProperty(NewName, de.DefaultValue);
            lbData.Items.Add(p);
            lbData.SelectedItem = p;
            _provider.AddProperty(p);
            PropertyChanged?.Invoke(this, new PropertyChangeEventArgs(p, _provider, PropertyChangeEventArgs.Change.Added, p.Name));
        }

        private void bDelete_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(UIResources.MSG_REMOVE, UIResources.CAP_WARNING, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
            {
                IObjectProperty p = lbData.SelectedItem as IObjectProperty;
                if (p != null)
                {
                    lbData.Items.Remove(p);
                    _provider.RemoveProperty(p);
                    PropertyChanged?.Invoke(this, new PropertyChangeEventArgs(p, _provider, PropertyChangeEventArgs.Change.Deleted, p.Name));
                }
            }
        }
    }
}
