﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Reflection;
using System.Windows.Forms;
using System.Threading;
using CellularAutomata;
using CellularAutomata.Interfaces;
using CellularAutomata.Automata;
using WinCA.Properties;
using WinCA.Interfaces;
using WinCA.Utils;

namespace WinCA.Forms
{
    public partial class frmAutomata : Form, ICAManager
    {
        private const int cCACOntrolsLocation = 14;

        private IMainForm _mainForm = null;
        private string _fileName = "";
        private ICANetwork _automata = null;
        private ICAStateManager _stateManager = null;
        private int _retard = 0;

        public frmAutomata()
        {
            InitializeComponent();
            Translate();
            GetData();
        }
        public frmAutomata(ICANetwork auto, string filename)
        {
            InitializeComponent();
            _fileName = filename;
            Translate();
            _automata = auto;
            GetData();
            cbCATypes.Enabled = false;
        }
        public frmAutomata(ICAStateManager mgr)
        {
            InitializeComponent();
            Translate();
            GetData();
            _stateManager = mgr;
            cbCATypes.SelectedIndex = 0;
        }

        #region ICAManager
        public ICAStateManager StateManager
        {
            get
            {
                return _stateManager;
            }
            set
            {
                _stateManager = value;
                if ((_stateManager != null) && (_automata != null))
                {
                    _automata.StateProvider = value.Provider;
                    _automata.VariableProvider = value.VariableProvider;
                    _automata.Initialize();
                }
            }
        }
        public ICAStateProvider StateProvider
        {
            get
            {
                if (_automata != null)
                {
                    return _automata.StateProvider;
                }
                return null;
            }
        }
        public ICANetwork Automata
        {
            get
            {
                return _automata;
            }
        }
        public IMainForm MainForm
        {
            get
            {
                return _mainForm;
            }
            set
            {
                _mainForm = value;
                if (_mainForm != null)
                {
                    _mainForm.OnStateManagerChanged += new StateManagerChangedEventHandler(StateManagerChanged);
                }
            }
        }
        #endregion
        private void Translate()
        {
            Text = string.Format(UIResources.TTL_RENDER, _fileName);
            bNeighbor.ToolTipText = UIResources.TTP_SHOWNB;
            bZoomIn.ToolTipText = UIResources.TTP_ZOOMIN;
            bZoomOut.ToolTipText = UIResources.TTP_ZOOMOUT;
            bSlow.ToolTipText = UIResources.TTP_SLOW;
            bFast.ToolTipText = UIResources.TTP_FAST;
            lCA.Text = UIResources.LAB_CATYPE;
            lCellProviders.Text = UIResources.LAB_CPTYPE;
            lOptimizer.Text = UIResources.LAB_OPTYPE;
            bPlay.ToolTipText = UIResources.TTP_PLAY;
            bPause.ToolTipText = UIResources.TTP_PAUSE;
            bStep.ToolTipText = UIResources.TTP_STEP;
            lStep.Text = "";
            bRestart.ToolTipText = UIResources.TTP_RTESTART;
            mnuFile.Text = UIResources.MNU_FILE;
            opSave.Text = UIResources.OPT_SAVECA;
            opSaveAs.Text = UIResources.OPT_SAVECAAS;
        }
        private void GetData()
        {
            cbCATypes.Items.Clear();
            cbCellProviders.Items.Clear();
            cbOptimizers.Items.Clear();
            cbOptimizers.Items.Add(UIResources.OPT_NOOPT);
            foreach (Type t in Assembly.GetAssembly(typeof(CAStub)).GetTypes())
            {
                if (t.GetInterface("ICANetwork") != null)
                {
                    cbCATypes.Items.Add(t.GetConstructor(Type.EmptyTypes).Invoke(null));
                }
                else if (t.GetInterface("ICACellProvider") != null)
                {
                    cbCellProviders.Items.Add(t.GetConstructor(Type.EmptyTypes).Invoke(null));
                }
                else if (t.GetInterface("ICAOptimizer") != null)
                {
                    cbOptimizers.Items.Add(t.GetConstructor(Type.EmptyTypes).Invoke(null));
                }
            }
            if (_automata != null)
            {
                foreach (object ca in cbCATypes.Items)
                {
                    if (ca.GetType() == _automata.GetType())
                    {
                        cbCATypes.SelectedItem = ca;
                        if (_automata.CellProvider != null)
                        {
                            foreach (object cp in cbCellProviders.Items)
                            {
                                if (cp.GetType() == _automata.CellProvider.GetType())
                                {
                                    cbCellProviders.SelectedItem = cp;
                                    break;
                                }
                            }
                        }
                        else
                        {
                            cbCellProviders.SelectedIndex = 0;
                        }
                        if (_automata.Optimizer != null)
                        {
                            foreach (object op in cbOptimizers.Items)
                            {
                                if (op.GetType() == _automata.Optimizer.GetType())
                                {
                                    cbOptimizers.SelectedItem = op;
                                    break;
                                }
                            }
                        }
                        else
                        {
                            foreach (object op in cbOptimizers.Items)
                            {
                                if (op is string)
                                {
                                    cbOptimizers.SelectedItem = op;
                                    break;
                                }
                            }
                        }
                        break;
                    }
                }
            }
            else
            {
                cbCellProviders.SelectedIndex = 0;
                cbOptimizers.SelectedIndex = 0;
            }
            cbCATypes.SelectedIndexChanged += new EventHandler(AutomataSelected);
            cbCellProviders.SelectedIndexChanged += new EventHandler(CellProviderSelected);
            cbOptimizers.SelectedIndexChanged += new EventHandler(OptimizerSelected);
        }
        private void InitializeAutomata()
        {
            if (_stateManager != null)
            {
                _automata.StateProvider = _stateManager.Provider;
                _automata.VariableProvider = _stateManager.VariableProvider;
            }
            if (cbCellProviders.SelectedItem != null)
            {
                _automata.CellProvider = cbCellProviders.SelectedItem as ICACellProvider;
            }
            scRender.Panel1.Controls.Add(_automata.Editor);
            _automata.ConfigureToolBar(tsCAControls, cCACOntrolsLocation, false);
            _automata.Editor.Dock = DockStyle.Fill;
            _automata.Initialize();
            OptimizerSelected(null, EventArgs.Empty);
        }
        private void SaveFile(string filename)
        {
            FileStream f = null;
            try
            {
                f = new FileStream(filename, FileMode.OpenOrCreate, FileAccess.Write, FileShare.None);
                BinaryFormatter fmt = new BinaryFormatter();
                fmt.Serialize(f, _automata);
                _fileName = filename;
                Text = string.Format(UIResources.TTL_RENDER, _fileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                if (f != null)
                {
                    f.Close();
                }
            }
        }
        private void CenterRender(int width, int height)
        {
            pbRender.Width = width;
            pbRender.Height = height;
            pbRender.Top = (scRender.Panel2.Height - height) / 2;
            pbRender.Left = (scRender.Panel2.Width - width) / 2;
        }
        private void SetImage(Bitmap bmp)
        {
            try
            {
                if (pbRender.Image == null)
                {
                    CenterRender(bmp.Width, bmp.Height);
                }
                Bitmap bold = pbRender.Image as Bitmap;
                if (bmp != null)
                {
                    pbRender.Size = new Size(bmp.Width, bmp.Height);
                }
                pbRender.Image = bmp;
                if (bold != null)
                {
                    bold.Dispose();
                }
            }
            catch
            {
            }
        }

        private void AutomataSelected(object sender, EventArgs e)
        {
            try
            {
                if (cbCATypes.SelectedItem != null)
                {
                    scRender.Panel1.Controls.Clear();
                    if (_automata != null)
                    {
                        _automata.ConfigureToolBar(tsCAControls, cCACOntrolsLocation, true);
                    }
                    _automata = cbCATypes.SelectedItem as ICANetwork;
                    if (_automata != null)
                    {
                        InitializeAutomata();
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        private void CellProviderSelected(object sender, EventArgs e)
        {
            if ((cbCellProviders.SelectedItem != null) && (_automata != null))
            {
                _automata.CellProvider = cbCellProviders.SelectedItem as ICACellProvider;
            }
        }
        private void OptimizerSelected(object sender, EventArgs e)
        {
            if ((cbOptimizers.SelectedItem != null) && (_automata != null))
            {
                if (cbOptimizers.SelectedItem is string)
                {
                    _automata.Optimizer = null;
                }
                else
                {
                    _automata.Optimizer = cbOptimizers.SelectedItem as ICAOptimizer;
                }
            }
        }
        private void StateManagerChanged(object sender, StateManagerEventArgs e)
        {
            switch (e.ChangeType)
            {
                case StateManagerEventArgs.Change.Removed:
                    if (e.StateManager == _stateManager)
                    {
                        _stateManager = null;
                    }
                    break;
            }
        }

        private void opSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(_fileName))
            {
                opSaveAs_Click(sender, e);
                return;
            }
            SaveFile(_fileName);
        }

        private void opSaveAs_Click(object sender, EventArgs e)
        {
            sfDlg.Filter = UIResources.FIL_CAUTO;
            if (sfDlg.ShowDialog() == DialogResult.OK)
            {
                SaveFile(sfDlg.FileName);
            }
        }

        private void scRender_Panel2_SizeChanged(object sender, EventArgs e)
        {
            CenterRender(pbRender.Width, pbRender.Height);
        }

        private void frmAutomata_Load(object sender, EventArgs e)
        {
            try
            {
                Icon = Icon.Clone() as Icon;
                if (_automata == null)
                {
                    cbCATypes.SelectedIndex = 0;
                    StateManager = _stateManager;
                }
                else
                {
                    InitializeAutomata();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void bRestart_Click(object sender, EventArgs e)
        {
            try
            {
                if (_automata != null)
                {
                    SetImage(_automata.Reset());
                    lStep.Text = string.Format(UIResources.LAB_STEP, _automata.ActualStep.ToString());
                    bPlay.Enabled = true;
                    bStep.Enabled = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void bStep_Click(object sender, EventArgs e)
        {
            try
            {
                if (_automata != null)
                {
                    SetImage(_automata.Step());
                    lStep.Text = string.Format(UIResources.LAB_STEP, _automata.ActualStep.ToString());
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void bZoomIn_Click(object sender, EventArgs e)
        {
            try
            {
                if (_automata != null)
                {
                    Bitmap img = _automata.ZoomIn();
                    SetImage(img);
                    if (img != null)
                    {
                        CenterRender(img.Width, img.Height);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void bZoomOut_Click(object sender, EventArgs e)
        {
            try
            {
                if (_automata != null)
                {
                    Bitmap img = _automata.ZoomOut();
                    SetImage(img);
                    if (img != null)
                    {
                        CenterRender(img.Width, img.Height);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void bgRunAutomata_DoWork(object sender, DoWorkEventArgs e)
        {
            while ((!bgRunAutomata.CancellationPending) && (_automata.ActualStep < int.MaxValue))
            {
                Bitmap img = _automata.Step();
                BeginInvoke((Action)delegate () {
                    SetImage(img);                    
                });
                bgRunAutomata.ReportProgress(0);
                if (_retard > 0)
                {
                    Thread.Sleep(_retard);
                }
            }
        }

        private void bgRunAutomata_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            lStep.Text = string.Format(UIResources.LAB_STEP, _automata.ActualStep.ToString());
        }

        private void bgRunAutomata_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message);
            }
            bStep.Enabled = _automata.ActualStep < int.MaxValue;
            bPlay.Enabled = _automata.ActualStep < int.MaxValue;
            bRestart.Enabled = true;
            bPause.Enabled = false;
        }

        private void bPlay_Click(object sender, EventArgs e)
        {
            try
            {
                if (_automata != null)
                {
                    bStep.Enabled = false;
                    bPause.Enabled = true;
                    bPlay.Enabled = false;
                    bRestart.Enabled = false;
                    bgRunAutomata.RunWorkerAsync();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void bPause_Click(object sender, EventArgs e)
        {
            if (bgRunAutomata.IsBusy)
            {
                bgRunAutomata.CancelAsync();
            }
        }

        private void bStateEditor_Click(object sender, EventArgs e)
        {
            try
            {
                if (_stateManager != null)
                {
                    if (_stateManager is Form)
                    {
                        (_stateManager as Form).BringToFront();
                    }
                }
                else
                {
                    StateSerializationContainer sts = new StateSerializationContainer()
                    {
                        StateProvider = _automata.StateProvider,
                        Properties = new PropertySerializationContainer()
                        {
                            ConstantProvider = _automata.StateProvider.PropertyProvider,
                            VariableProvider = _automata.VariableProvider
                        }
                    };
                    _stateManager = MainForm.CreateStateManager(sts, "");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void bSlow_Click(object sender, EventArgs e)
        {
            if (_retard < 1000)
            {
                _retard += 20;
            }
        }

        private void bFast_Click(object sender, EventArgs e)
        {
            if (_retard > 0)
            {
                _retard -= 20;
            }
        }

        private void frmAutomata_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (bgRunAutomata.IsBusy)
            {
                bgRunAutomata.CancelAsync();
            }
        }

        private void bNeighbor_Click(object sender, EventArgs e)
        {
            bNeighbor.Checked = !bNeighbor.Checked;
            scRender.Panel1Collapsed = !bNeighbor.Checked;
        }

        private void frmAutomata_FormClosed(object sender, FormClosedEventArgs e)
        {
            _mainForm.OnStateManagerChanged -= new StateManagerChangedEventHandler(StateManagerChanged);
        }
    }
}
