﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using System.Configuration;
using System.Windows.Forms;
using CommonObjects.Interfaces;
using CommonObjects.Data;
using WinCA.Properties;
using WinCA.Interfaces;
using WinCA.Utils;
using WinCA.Controls;

namespace WinCA.Forms
{
    public partial class frmProperties : Form, IPropertyManager
    {
        private IPropertyProvider _constantProvider = null;
        private IPropertyProvider _variableProvider = null;
        private string _fileName = "";
        public frmProperties()
        {
            InitializeComponent();
            Translate();
            Type tp = Type.GetType(ConfigurationManager.AppSettings["defaultPropertyProviderType"]);
            _constantProvider = tp.GetConstructor(Type.EmptyTypes).Invoke(null) as IPropertyProvider;
            _constantProvider.DefaultPropertyType = typeof(Constant);
            ConstantProvider = _constantProvider;
            _variableProvider = tp.GetConstructor(Type.EmptyTypes).Invoke(null) as IPropertyProvider;
            _variableProvider.DefaultPropertyType = typeof(Variable);
            VariableProvider = _variableProvider;
        }
        public frmProperties(string filename)
        {
            InitializeComponent();
            OpenFile(filename);
            Translate();
        }
        public frmProperties(PropertySerializationContainer lp, string filename)
        {
            InitializeComponent();
            ConstantProvider = lp.ConstantProvider;
            VariableProvider = lp.VariableProvider;
            _fileName = filename;
            Translate();
        }
        #region IPropertyManager
        public event PropertyChangeEventHandler PropertyChanged = null;
        public IPropertyProvider ConstantProvider
        {
            get
            {
                return _constantProvider;
            }
            set
            {
                _constantProvider = value;
                veConstants.Provider = value;
            }
        }
        public IPropertyProvider VariableProvider
        {
            get
            {
                return _variableProvider;
            }
            set
            {
                _variableProvider = value;
                veVariables.Provider = value;
            }
        }
        public IMainForm MainForm { get; set; }
        #endregion
        public override string ToString()
        {
            return _constantProvider.Name;
        }
        private void Translate()
        {
            Text = string.Format(UIResources.TTL_PROPERTIES, _fileName);
            veConstants.ValueName = UIResources.LAB_CONSTANT;
            veVariables.ValueName = UIResources.LAB_VARIABLE;
            opFile.Text = UIResources.MNU_FILE;
        }
        private void OpenFile(string filename)
        {
            FileStream f = null;
            try
            {
                f = new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.Read);
                BinaryFormatter fmt = new BinaryFormatter();
                PropertySerializationContainer ls = fmt.Deserialize(f) as PropertySerializationContainer;
                ConstantProvider = ls.ConstantProvider;
                VariableProvider = ls.VariableProvider;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                if (f != null)
                {
                    f.Close();
                }
            }
        }
        private void SaveFile(string filename)
        {
            FileStream f = null;
            try
            {
                f = new FileStream(filename, FileMode.OpenOrCreate, FileAccess.Write, FileShare.None);
                BinaryFormatter fmt = new BinaryFormatter();
                PropertySerializationContainer ls = new PropertySerializationContainer();
                _constantProvider.Name = filename;
                ls.ConstantProvider = _constantProvider;
                _variableProvider.Name = filename;
                ls.VariableProvider = _variableProvider;
                fmt.Serialize(f, ls);
                _fileName = filename;
                Text = string.Format(UIResources.TTL_PROPERTIES, _fileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                if (f != null)
                {
                    f.Close();
                }
            }
        }

        private void opSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(_fileName))
            {
                opSaveAs_Click(sender, e);
                return;
            }
            SaveFile(_fileName);
        }

        private void opSaveAs_Click(object sender, EventArgs e)
        {
            sfDlg.Filter = UIResources.FIL_PROPERTIES;
            if (sfDlg.ShowDialog() == DialogResult.OK)
            {
                SaveFile(sfDlg.FileName);
            }
        }

        private void veConstants_PropertyChanged(object sender, PropertyChangeEventArgs e)
        {
            PropertyChanged?.Invoke(this, e);
        }

        private void frmProperties_Load(object sender, EventArgs e)
        {
            Icon = Icon.Clone() as Icon;
        }
    }
}
