﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.IO;
using System.Windows.Forms;
using CommonObjects.Expressions;
using CellularAutomata.Interfaces;
using CellularAutomata.Expressions;
using BNFUP.Rules;
using BNFUP.Interfaces;
using BNFUP.Exceptions;
using WinCA.Properties;
using WinCA.Interfaces;
using WinCA.Utils;
using WinCA.Controls;

namespace WinCA.Forms
{
    public partial class frmTransitions : Form, ITransitionManager
    {
        private IMainForm _mainForm = null;
        private string _fileName = "";
        private ICAStateManager _manager = null;
        private ICAManager _caManager = null;
        private ICompilableObject _object = null;
        private List<string> _messages = null;

        public frmTransitions()
        {
            InitializeComponent();
            Translate();
        }
        public frmTransitions(ICAStateManager mgr, string filename)
        {
            InitializeComponent();
            _manager = mgr;
            _fileName = filename;
            Translate();
        }
        #region ITransitionManager
        public string UID
        {
            get
            {
                if (_manager != null)
                {
                    return _manager.Provider.UID;
                }
                return "";
            }
        }
        public ICAManager AutomataManager
        {
            get
            {
                return _caManager;
            }
            set
            {
                _caManager = value;
            }
        }
        public IMainForm MainForm
        {
            get
            {
                return _mainForm;
            }
            set
            {
                _mainForm = value;
                _mainForm.OnStateManagerChanged += new StateManagerChangedEventHandler(StateManagerChanged);
                cbStates.Items.Clear();
                foreach (ICAStateManager sm in _mainForm.StateManagers)
                {
                    cbStates.Items.Add(sm);
                }
                if (_manager != null)
                {
                    if (!SelectStateManager(_manager.Provider.UID))
                    {
                        StateSerializationContainer sts = new StateSerializationContainer();
                        sts.StateProvider = _manager.Provider;
                        sts.Properties = new PropertySerializationContainer();
                        sts.Properties.ConstantProvider = _manager.Provider.PropertyProvider;
                        sts.Properties.VariableProvider = _manager.VariableProvider;
                        _mainForm.CreateStateManager(sts, _manager.Provider.Name);
                        SelectStateManager(_manager.Provider.UID);
                    }
                    tmPanel.Provider = _manager.Provider;
                }
                else
                {
                    if (cbStates.Items.Count == 0)
                    {
                        _mainForm.CreateStateManager(null, "");
                    }
                    cbStates.SelectedIndex = 0;
                    tmPanel.Provider = null;
                }
                GetStates();
            }
        }
        #endregion
        private void Translate()
        {
            Text = string.Format(UIResources.TTL_TRANSITIONS, _fileName);
            lStates.Text = UIResources.LAB_STATEMGR;
            bStateEditor.ToolTipText = UIResources.TTP_SEDITOR;
            bBuild.ToolTipText = UIResources.TTP_BUILD;
            bTest.ToolTipText = UIResources.TTP_TESTEXPR;
            bScript.ToolTipText = UIResources.TTP_COMPILATIONSC;
        }
        private void CompilationMessage(string msg)
        {
            _messages.Add(msg);
        }
        private void GetStates()
        {
            tvTransitions.Nodes.Clear();
            if (_manager != null)
            {
                foreach (ICAState st in _manager.Provider.States)
                {
                    TreeNode node = new TreeNode(st.Name);
                    node.Tag = st;
                    foreach (string t in st.Transitions)
                    {
                        node.Nodes.Add(t);
                    }
                    tvTransitions.Nodes.Add(node);
                }
            }
        }
        private bool SelectStateManager(string uid)
        {
            for (int ix = 0; ix < cbStates.Items.Count; ix++)
            {
                ICAStateManager pm = cbStates.Items[ix] as ICAStateManager;
                if (pm.Provider.UID == uid)
                {
                    cbStates.SelectedIndex = ix;
                    return true;
                }
            }
            cbStates.SelectedIndex = -1;
            return false;
        }
        private void StateManagerChanged(object sender, StateManagerEventArgs e)
        {
            switch (e.ChangeType)
            {
                case StateManagerEventArgs.Change.Added:
                    cbStates.Items.Add(e.StateManager);
                    break;
                case StateManagerEventArgs.Change.Removed:
                    cbStates.Items.Remove(e.StateManager);
                    SelectStateManager(_manager.Provider.UID);
                    break;
            }
        }

        private void frmTransitions_FormClosed(object sender, FormClosedEventArgs e)
        {
            _mainForm.OnStateManagerChanged -= new StateManagerChangedEventHandler(StateManagerChanged);
        }

        private void tmPanel_OnDeleteTransition(object sender, TransitionEventArgs e)
        {
            try
            {
                foreach (TreeNode node in tvTransitions.Nodes)
                {
                    if (node.Text == e.From)
                    {
                        foreach (TreeNode nt in node.Nodes)
                        {
                            if (nt.Text == e.To)
                            {
                                if (tvTransitions.SelectedNode == nt)
                                {
                                    txtExpression.Text = "";
                                }
                                node.Nodes.Remove(nt);
                                return;
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void tmPanel_OnEditTransition(object sender, TransitionEventArgs e)
        {
            try
            {
                foreach (TreeNode node in tvTransitions.Nodes)
                {
                    if (node.Text == e.From)
                    {
                        TreeNode nedit = null;
                        foreach (TreeNode nt in node.Nodes)
                        {
                            if (nt.Text == e.To)
                            {
                                nedit = nt;
                                break;
                            }
                        }
                        if (nedit == null)
                        {
                            nedit = node.Nodes.Add(e.To);
                        }
                        nedit.Expand();
                        tvTransitions.SelectedNode = nedit;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void tvTransitions_AfterSelect(object sender, TreeViewEventArgs e)
        {
            try
            {
                if ((tvTransitions.SelectedNode != null) && (tvTransitions.SelectedNode.Tag == null))
                {
                    txtExpression.ReadOnly = false;
                    ICAState st = tvTransitions.SelectedNode.Parent.Tag as ICAState;
                    if (st != null)
                    {
                        txtExpression.Text = st.GetTransition(tvTransitions.SelectedNode.Text);
                    }
                    tmPanel.Refresh();
                }
                else
                {
                    txtExpression.Text = "";
                    txtExpression.ReadOnly = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void tvTransitions_BeforeSelect(object sender, TreeViewCancelEventArgs e)
        {
            try
            {
                if ((tvTransitions.SelectedNode != null) && (tvTransitions.SelectedNode.Tag == null))
                {
                    ICAState st = tvTransitions.SelectedNode.Parent.Tag as ICAState;
                    if (st != null)
                    {
                        st.SetTransition(tvTransitions.SelectedNode.Text, txtExpression.Text);
                        if (string.IsNullOrEmpty(txtExpression.Text))
                        {
                            tvTransitions.SelectedNode.Parent.Nodes.Remove(tvTransitions.SelectedNode);
                        }
                    }
                    tmPanel.Refresh();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void txtExpression_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if ((tvTransitions.SelectedNode != null) && (tvTransitions.SelectedNode.Tag == null))
                {
                    ICAState st = tvTransitions.SelectedNode.Parent.Tag as ICAState;
                    if (st != null)
                    {
                        string txt = st.GetTransition(tvTransitions.SelectedNode.Text);
                        st.SetTransition(tvTransitions.SelectedNode.Text, txtExpression.Text);
                        if ((string.IsNullOrEmpty(txt) && !string.IsNullOrEmpty(txtExpression.Text) )
                            || (string.IsNullOrEmpty(txtExpression.Text) && !string.IsNullOrEmpty(txt)))
                        {
                            tmPanel.Refresh();
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void bBuild_Click(object sender, EventArgs e)
        {
            try
            {
                _messages = new List<string>();
                ExpressionFactory factory = new ExpressionFactory();
                RuleTable rt = factory.Compiler;
                rt.DebugMode = true;
                rt.OnCompilationFeedback += new CompilerEventHandler(CompilationMessage);
                StringReader sr = new StringReader(txtExpression.Text);
                _object = rt.Build(sr);
                bTest.Enabled = (_object != null);
                MessageBox.Show(UIResources.MSG_COMPILED);
            }
            catch (CodeSyntaxErrorException sex)
            {
                txtExpression.Select(sex.Position, 1);
                MessageBox.Show(sex.Message);
                txtExpression.Focus();
            }
            catch (InvalidTokenException tex)
            {
                txtExpression.Select(tex.Position, 1);
                MessageBox.Show(string.Join(": ", new string[] { tex.Message, tex.Token }));
                txtExpression.Focus();
            }
            catch (UnknownObjectKeyException uex)
            {
                MessageBox.Show(string.Join(": ", new string[] { uex.Message, uex.Key }));
                txtExpression.Focus();
            }
            catch (RejectedObjectException rex)
            {
                MessageBox.Show(string.Join(", ", new string[] { rex.Parent, rex.Message, rex.Child }));
                txtExpression.Focus();
            }
            catch (NotMatchingCharacterException cex)
            {
                txtExpression.Select(cex.Position, 1);
                MessageBox.Show(cex.Message);
                txtExpression.Focus();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                if (!string.IsNullOrEmpty(txtExpression.Text))
                {
                    txtExpression.Select(txtExpression.Text.Length - 1, 1);
                }
            }
            bScript.Enabled = true;
        }

        private void bTest_Click(object sender, EventArgs e)
        {
            try
            {
                frmTest frm = new frmTest();
                frm.StateProvider = _manager.Provider;
                frm.VariableProvider = _manager.VariableProvider;
                frm.Expression = _object as ExprBase;
                frm.ShowDialog();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void bScript_Click(object sender, EventArgs e)
        {
            frmMessages fmsg = new frmMessages();
            fmsg.SetMessages(_messages);
            fmsg.ShowDialog(this);
        }

        private void bStateEditor_Click(object sender, EventArgs e)
        {
            try
            {
                if (_manager is Form)
                {
                    (_manager as Form).BringToFront();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void frmTransitions_Load(object sender, EventArgs e)
        {
            Icon = Icon.Clone() as Icon;
        }
    }
}
