﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CellularAutomata.Interfaces;
using WinCA.Utils;

namespace WinCA.Interfaces
{
    public class PropertyManagerEventArgs : EventArgs
    {
        public enum Change
        {
            Added, Removed
        }
        public PropertyManagerEventArgs(IPropertyManager pm, Change chg)
        {
            PropertyManager = pm;
            ChangeType = chg;
        }
        public IPropertyManager PropertyManager { get; private set; }
        public Change ChangeType { get; private set; }
    }
    public delegate void PropertyManagerChangedEventHandler(object sender, PropertyManagerEventArgs e);
    public class StateManagerEventArgs : EventArgs
    {
        public enum Change
        {
            Added, Removed
        }
        public StateManagerEventArgs(ICAStateManager sm, Change chg)
        {
            StateManager = sm;
            ChangeType = chg;
        }
        public ICAStateManager StateManager { get; private set; }
        public Change ChangeType { get; private set; }
    }
    public delegate void StateManagerChangedEventHandler(object sender, StateManagerEventArgs e);
    public interface IMainForm
    {
        event PropertyManagerChangedEventHandler OnPropertyManagerChanged;
        event StateManagerChangedEventHandler OnStateManagerChanged;
        IEnumerable<IPropertyManager> PropertyManagers { get; }
        IEnumerable<ICAStateManager> StateManagers { get; }
        IPropertyManager CreatePropertyManager(PropertySerializationContainer pms, string filename);                
        ICAStateManager CreateStateManager(StateSerializationContainer sms, string filename);
        ITransitionManager CreateTransitionManager(ICAStateManager mgr, string filename);
        ICAManager CreateCAManger(ICAStateManager mgr);
        ICAManager CreateCAManger(ICANetwork ca, string filename);
    }
}
