﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Diagnostics;
using System.IO;
using System.Globalization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Windows.Forms;
using CellularAutomata.Interfaces;
using WinCA.Properties;
using WinCA.Forms;
using WinCA.Interfaces;
using WinCA.Utils;

namespace WinCA
{
    public partial class MainForm : Form, IMainForm
    {
        public MainForm()
        {
            InitializeComponent();
            Text = UIResources.TTL_MAIN;
            mnuFile.Text = UIResources.MNU_FILE;
            opNew.Text = UIResources.OPT_NEW;
            opPropertySet.Text = UIResources.OPT_NEWPROPERTYSET;
            opStateSet.Text = UIResources.OPT_NEWSTATESET;
            opOpen.Text = UIResources.OPT_OPEN;
            opExit.Text = UIResources.OPT_EXIT;
            mnuWindow.Text = UIResources.MNU_WINDOW;
            mnuHelp.Text = UIResources.MNU_HELP;
        }
        #region IMainForm
        public event PropertyManagerChangedEventHandler OnPropertyManagerChanged = null;
        public event StateManagerChangedEventHandler OnStateManagerChanged = null;
        public IEnumerable<IPropertyManager> PropertyManagers
        {
            get
            {
                foreach (Form frm in MdiChildren)
                {
                    if (frm is IPropertyManager)
                    {
                        yield return frm as IPropertyManager;
                    }
                }
            }
        }
        public IEnumerable<ICAStateManager> StateManagers
        {
            get
            {
                foreach (Form frm in MdiChildren)
                {
                    if (frm is ICAStateManager)
                    {
                        yield return frm as ICAStateManager;
                    }
                }
            }
        }
        public IPropertyManager CreatePropertyManager(PropertySerializationContainer pms, string filename)
        {
            frmProperties frm = null;
            if (pms == null)
            {
                frm = new frmProperties();
                frm.MdiParent = this;
                frm.MainForm = this;             
                frm.Show();
            }
            else
            {
                foreach (Form f in MdiChildren)
                {
                    IPropertyManager pm = f as IPropertyManager;
                    if (pm != null)
                    {
                        if (pm.ConstantProvider.UID == pms.ConstantProvider.UID)
                        {
                            f.BringToFront();
                            return pm;
                        }
                    }
                }
                frm = new frmProperties(pms, filename);
                frm.MdiParent = this;
                frm.MainForm = this;
                frm.Show();                
            }
            frm.FormClosed += new FormClosedEventHandler(PropertyManagerClosed);
            OnPropertyManagerChanged?.Invoke(this, new PropertyManagerEventArgs(frm, PropertyManagerEventArgs.Change.Added));
            return frm;
        }        
        public ICAStateManager CreateStateManager(StateSerializationContainer sms, string filename)
        {
            frmStates frm = null;
            if (sms == null)
            {
                frm = new frmStates();
                frm.MdiParent = this;
                frm.MainForm = this;
                frm.Show();
            }
            else
            {
                foreach (Form f in MdiChildren)
                {
                    ICAStateManager sm = f as ICAStateManager;
                    if (sm != null)
                    {
                        if (sm.Provider.UID == sms.StateProvider.UID)
                        {
                            f.BringToFront();
                            return sm;
                        }
                    }
                }
                frm = new frmStates(sms, filename);
                frm.MdiParent = this;
                frm.MainForm = this;
                frm.Show();
            }
            frm.FormClosed += new FormClosedEventHandler(StateManagerClosed);
            OnStateManagerChanged?.Invoke(this, new StateManagerEventArgs(frm, StateManagerEventArgs.Change.Added));
            return frm;
        }
        public ITransitionManager CreateTransitionManager(ICAStateManager mgr, string filename)
        {
            frmTransitions frm = null;
            if (mgr == null)
            {
                frm = new frmTransitions();
                frm.MdiParent = this;
                frm.MainForm = this;
                frm.Show();
            }
            else
            {
                foreach (Form f in MdiChildren)
                {
                    frmTransitions tm = f as frmTransitions;
                    if (tm != null)
                    {
                        if (tm.UID == mgr.Provider.UID)
                        {
                            f.BringToFront();
                            return tm;
                        }
                    }
                }
                frm = new frmTransitions(mgr, filename);
                frm.MdiParent = this;
                frm.MainForm = this;
                frm.Show();
            }
            return frm;
        }
        public ICAManager CreateCAManger(ICAStateManager mgr)
        {
            frmAutomata frm = null;
            if (mgr == null)
            {
                frm = new frmAutomata();
                frm.MdiParent = this;
                frm.MainForm = this;
                frm.Show();
            }
            else
            {
                foreach (Form f in MdiChildren)
                {
                    frmAutomata tm = f as frmAutomata;
                    if ((tm != null) && (tm.StateProvider != null))
                    {
                        if (tm.StateProvider.UID == mgr.Provider.UID)
                        {
                            f.BringToFront();
                            return tm;
                        }
                    }
                }
                frm = new frmAutomata(mgr);
                frm.MdiParent = this;
                frm.MainForm = this;
                frm.Show();
            }
            return frm;
        }
        public ICAManager CreateCAManger(ICANetwork ca, string filename)
        {
            frmAutomata frm = null;
            if (ca == null)
            {
                frm = new frmAutomata();
                frm.MdiParent = this;
                frm.MainForm = this;
                frm.Show();
            }
            else
            {
                frm = new frmAutomata(ca, filename);
                frm.MdiParent = this;
                frm.MainForm = this;
                frm.Show();
            }
            return frm;
        }
        #endregion
        private void PropertyManagerClosed(object sender, FormClosedEventArgs e)
        {
            OnPropertyManagerChanged?.Invoke(this, new PropertyManagerEventArgs(sender as IPropertyManager, PropertyManagerEventArgs.Change.Removed));
        }
        private void StateManagerClosed(object sender, FormClosedEventArgs e)
        {
            OnStateManagerChanged?.Invoke(this, new StateManagerEventArgs(sender as ICAStateManager, StateManagerEventArgs.Change.Removed));
        }
        private void opPropertySet_Click(object sender, EventArgs e)
        {
            try
            {
                CreatePropertyManager(null, "");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void opOpen_Click(object sender, EventArgs e)
        {
            ofDlg.Filter = UIResources.FIL_ALLFILES;
            if (ofDlg.ShowDialog() == DialogResult.OK)
            {
                FileStream f = null;
                try
                {
                    f = new FileStream(ofDlg.FileName, FileMode.Open, FileAccess.Read, FileShare.Read);
                    BinaryFormatter fmt = new BinaryFormatter();
                    object obj = fmt.Deserialize(f);
                    if (obj is PropertySerializationContainer)
                    {
                        CreatePropertyManager(obj as PropertySerializationContainer, ofDlg.FileName);
                    }
                    else if (obj is StateSerializationContainer)
                    {
                        CreateStateManager(obj as StateSerializationContainer, ofDlg.FileName);
                    }
                    else if (obj is ICANetwork)
                    {
                        CreateCAManger(obj as ICANetwork, ofDlg.FileName);
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
                finally
                {
                    if (f != null)
                    {
                        f.Close();
                    }
                }
            }
        }

        private void opStateSet_Click(object sender, EventArgs e)
        {
            try
            {
                CreateStateManager(null, "");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void opExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void mnuHelp_Click(object sender, EventArgs e)
        {
            try
            {
                Process p = new Process();
                p.StartInfo.FileName = UIResources.URL_HELP;
                p.Start();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
    }
}
